import unittest
import sys
from io import StringIO

import mock

from tests.mock_data import *
import maccli.command_cli

DEFAULT_DEPLOYMENT = "testing"
DEFAULT_PROVIDER = "default"
DEFAULT_LIFESPAN = 90
DEFAULT_BRANCH = "master"
DEFAULT_RELEASE = "any"
DEFAULT_PORT = [22]
DEFAULT_NET = ""


class AuthTestCase(unittest.TestCase):
    def setUp(self):
        self.stdout = sys.stdout
        sys.stdout = self.buf = StringIO()
        # pass

    def tearDown(self):
        sys.stdout = self.stdout
        # pass

    @mock.patch('maccli.service.instance.list_instances')
    def test_instance_list(self, mock_list_instances):
        mock_list_instances.return_value = MOCK_INSTANCE_LIST_JSON
        maccli.command_cli.instance_list()

    def test_instance_create_no_args(self):
        maccli.command_cli.instance_create(None, None, DEFAULT_DEPLOYMENT, None, None, DEFAULT_PROVIDER,
                                           DEFAULT_RELEASE, None,
                                           DEFAULT_BRANCH, None, DEFAULT_LIFESPAN, None, None, DEFAULT_PORT,
                                           DEFAULT_NET)
        out = self.buf.getvalue()
        self.assertEqual(' '.join(OUTPUT_CREATE_INSTANCE_NO_INPUT.split()), ' '.join(out.split()))

    @mock.patch('maccli.service.provider.list_locations')
    def test_instance_create_cookbook(self, mock_list_locations):
        mock_list_locations.return_value = MOCK_LOCATION_LIST_JSON
        maccli.command_cli.instance_create("cookbook_tag", None, DEFAULT_DEPLOYMENT, None, None, DEFAULT_PROVIDER,
                                           DEFAULT_RELEASE, None, DEFAULT_BRANCH, None, DEFAULT_LIFESPAN, None, None,
                                           DEFAULT_PORT, DEFAULT_NET)
        out = self.buf.getvalue()
        self.assertEqual(' '.join(OUTPUT_CREATE_INSTANCE_ONLY_CONFIGURATION.split()), ' '.join(out.split()))

    @mock.patch('maccli.service.provider.list_locations')
    def test_instance_create_cookbook_no_locations(self, mock_list_locations):
        mock_list_locations.return_value = MOCK_LOGIN_JSON_EMPTY
        maccli.command_cli.instance_create("cookbook_tag", None, DEFAULT_DEPLOYMENT, None, None, DEFAULT_PROVIDER,
                                           DEFAULT_RELEASE, None, DEFAULT_BRANCH, None, DEFAULT_LIFESPAN, None, None,
                                           DEFAULT_PORT, DEFAULT_NET)
        out = self.buf.getvalue()
        self.assertEqual(' '.join(OUTPUT_CREATE_INSTANCE_ONLY_CONFIGURATION_NO_LOCATIONS.split()),
                         ' '.join(out.split()))

    @mock.patch('maccli.service.provider.list_hardwares')
    def test_instance_create_production_cookbook_no_hardware(self, mock_list_hardwares):
        mock_list_hardwares.return_value = MOCK_HARDWARE_LIST_JSON
        maccli.command_cli.instance_create("cookbook_tag", None, "production", "sfo1", None, DEFAULT_PROVIDER,
                                           DEFAULT_RELEASE, None, DEFAULT_BRANCH, None, DEFAULT_LIFESPAN, None, None,
                                           DEFAULT_PORT, DEFAULT_NET)
        out = self.buf.getvalue()
        self.assertEqual(' '.join(OUTPUT_CREATE_INSTANCE_PRODUCTION_NO_HARDWARE.split()), ' '.join(out.split()))

    @mock.patch('maccli.service.instance.create_instance')
    def test_instance_create_development(self, mock_instance):
        mock_instance.return_value = MOCK_INSTANCE_CREATE_TESTING_OK_JSON
        maccli.command_cli.instance_create("cookbook_tag", None, DEFAULT_DEPLOYMENT, "sfo1", None, DEFAULT_PROVIDER,
                                           DEFAULT_RELEASE, None, DEFAULT_BRANCH, None, DEFAULT_LIFESPAN, None, None,
                                           DEFAULT_PORT, DEFAULT_NET)
        out = self.buf.getvalue()
        self.assertEqual(' '.join(OUTPUT_CREATE_INSTANCE_TESTING_OK.split()), ' '.join(out.split()))

    @mock.patch('maccli.service.instance.create_instance')
    def test_instance_create_production(self, mock_instance):
        mock_instance.return_value = MOCK_INSTANCE_CREATE_PRODUCTION_OK_JSON
        maccli.command_cli.instance_create("cookbook_tag", None, "production", "sfo1", None, DEFAULT_PROVIDER,
                                           DEFAULT_RELEASE, None, DEFAULT_BRANCH, "512mb", DEFAULT_LIFESPAN, None, None,
                                           DEFAULT_PORT, DEFAULT_NET)
        out = self.buf.getvalue()
        self.assertEqual(' '.join(OUTPUT_CREATE_INSTANCE_PRODUCTION_OK.split()), ' '.join(out.split()))

    @mock.patch('maccli.service.configuration.search_configurations')
    def test_configuration_search(self, mock):
        mock.return_value = MOCK_CONFIGURATION_SEARCH_JSON
        maccli.command_cli.configuration_search(None, False)
        out = self.buf.getvalue()
        self.assertEqual(' '.join(OUTPUT_CONFIGURATION_SEARCH.split()), ' '.join(out.split()))

    @mock.patch('maccli.service.configuration.search_configurations')
    def test_configuration_search_keywords(self, mock):
        mock.return_value = MOCK_CONFIGURATION_SEARCH_JSON
        maccli.command_cli.configuration_search(None, True)
        out = self.buf.getvalue()
        self.assertEqual(' '.join(OUTPUT_CONFIGURATION_SEARCH_KEYWORDS.split()), ' '.join(out.split()))

    @mock.patch('maccli.service.configuration.list_configurations')
    def test_configuration_list(self, mock):
        mock.return_value = MOCK_CONFIGURATION_LIST_JSON
        maccli.command_cli.configuration_list()
        out = self.buf.getvalue()
        self.assertEqual(' '.join(OUTPUT_CONFIGURATION_LIST.split()), ' '.join(out.split()))

    @mock.patch('maccli.view.view_generic.general_help')
    def test_help(self, mock):
        maccli.command_cli.help()
        self.assertTrue(mock.called)

    @mock.patch('maccli.view.view_instance.show_instance_destroy_help')
    def test_instance_destroy_help(self, mock):
        maccli.command_cli.instance_destroy_help()
        self.assertTrue(mock.called)

    @mock.patch('maccli.view.view_instance.show_instance_ssh_help')
    def test_instance_ssh_help(self, mock):
        maccli.command_cli.instance_ssh_help()
        self.assertTrue(mock.called)

    @mock.patch('maccli.view.view_instance.show_instance_help')
    def test_instance_help(self, mock):
        maccli.command_cli.instance_help()
        self.assertTrue(mock.called)

    @mock.patch('maccli.view.view_cookbook.show_configurations_help')
    def test_instance_help(self, mock):
        maccli.command_cli.configuration_help()
        self.assertTrue(mock.called)



import sys

# define colors
BLACK, RED, GREEN, YELLOW, BLUE, MAGENTA, CYAN, WHITE = range(8)


def show(string=""):
    if type(string) == list:
        for s in string:
            print(s)
    else:
        print(string.strip())


def showc(string, colour=WHITE):
    if _has_colours(sys.stdout):
        seq = "\x1b[1;%dm" % (30 + colour) + string + "\x1b[0m"
        sys.stdout.write(seq)
    else:
        sys.stdout.write(string)


def show_error(string):
    print(string, file=sys.stderr)


def clear():
    print(chr(27) + "[2J")


def general_help():
    print("Welcome to the beta version of Manageacloud cli.")
    print("For more information, please visit https://manageacloud.com")
    print("")
    print("To display the available options:")
    print("")
    print("    mac -h")
    print("")


def header(text, char=None):
    if char is None:
        char = "-"
    print(text)
    print(char * len(text))
    print()


def cmd_error(command, rc, stdout, stderr):
    show_error("Script exited with code %s" % rc)
    show_error(command)
    show_error("Error details: %s" % stderr)
    if stdout != "":
        show_error("Extra output: %s " % stdout)


def _has_colours(stream):
    if not hasattr(stream, "isatty"):
        return False
    if not stream.isatty():
        return False  # auto color only on TTYs
    try:
        import curses

        curses.setupterm()
        return curses.tigetnum("colors") > 2
    except:
        # guess false in case of error
        return False


def show_provider_help():
    print("")
    print("Show more help:")
    print("")
    print("    mac provider -h")
    print("")



import unittest
import sys
from io import StringIO

import mock

from tests.mock_data import *
import maccli.dao.api_configuration


DEFAULT_CONFIGURATION = "cookbook_tag"
DEFAULT_PROVIDER = "manageacloud"
DEFAULT_LOCATION = "sfo1"


class AuthTestCase(unittest.TestCase):
    def setUp(self):
        #self.stdout = sys.stdout
        #sys.stdout = self.buf = StringIO()
        pass

    def tearDown(self):
        #sys.stdout = self.stdout
        pass

    @mock.patch('maccli.helper.http.send_request')
    def test_search_no_params(self, mock):
        mock.return_value = (200, MOCK_CONFIGURATION_SEARCH_JSON, MOCK_CONFIGURATION_SEARCH_JSON_RAW)
        status_code, json_response = maccli.dao.api_configuration.search_public_configuration(None)
        mock.assert_called_once_with("GET", "/configuration/search")
        self.assertEqual(json_response, MOCK_CONFIGURATION_SEARCH_JSON)
        self.assertEqual(status_code, 200)

    @mock.patch('maccli.helper.http.send_request')
    def test_search_one_param(self, mock):
        keywords = ['one']
        mock.return_value = (200, MOCK_CONFIGURATION_SEARCH_JSON, MOCK_CONFIGURATION_SEARCH_JSON_RAW)
        status_code, json_response = maccli.dao.api_configuration.search_public_configuration(keywords)
        mock.assert_called_once_with("GET", "/configuration/search?keyword=one")
        self.assertEqual(json_response, MOCK_CONFIGURATION_SEARCH_JSON)
        self.assertEqual(status_code, 200)

    @mock.patch('maccli.helper.http.send_request')
    def test_search_several_param(self, mock):
        keywords = ['one', 'two']
        mock.return_value = (200, MOCK_CONFIGURATION_SEARCH_JSON, MOCK_CONFIGURATION_SEARCH_JSON_RAW)
        status_code, json_response = maccli.dao.api_configuration.search_public_configuration(keywords)
        mock.assert_called_once_with("GET", "/configuration/search?keyword=one&keyword=two")
        self.assertEqual(json_response, MOCK_CONFIGURATION_SEARCH_JSON)
        self.assertEqual(status_code, 200)


CONFIG_FILE = '.manageacloud'

MAC_FILE = '.manageacloud'
AUTH_SECTION = 'auth'
USER_OPTION = 'user'
APIKEY_OPTION = 'apikey'
AUTH_ERROR = 'auth_error'
NO_ERROR = 'no_error'

RELEASE_ANY = "any"
CONFIGURATION_FAILED = "Configuration Error"
CREATION_FAILED = "Creation failed"

AUTH_ERROR_EXIT_CODE = 2
EXCEPTION_EXIT_CODE = 3

MACFILE_ON_FAILURE_DESTROY_ALL = "destroy_all"
MACFILE_ON_FAILURE_DESTROY_OTHERS = "destroy_others"


from io import StringIO
import os
import unittest
import sys
import yaml

import mock

from tests.mock_data import *
import maccli.service.macfile
import maccli.helper.macfile
import maccli.dao.inheritance
from maccli.helper.exception import MacParseEnvException, MacParseParamException


class MacfileServiceTestCase(unittest.TestCase):

    def setUp(self):
        #self.stdout = sys.stdout
        #sys.stdout = self.buf = StringIO()

        if os.getcwd().endswith("/tests"):
            self.mock_path = "mock"
        else:
            self.mock_path = "tests/mock"

    def tearDown(self):
        #sys.stdout = self.stdout
        pass

    def test_open_file(self):
        contents = maccli.helper.macfile.load_macfile("%s/aws-medium-pgbench.macfile" % self.mock_path)
        macfile_raw = maccli.dao.inheritance.resolve_inheritance(contents, [])
        root, roles, infrastructures, _, _ = maccli.service.macfile.parse_macfile(macfile_raw)
        self.assertEquals(yaml.dump(roles, default_flow_style=False), yaml.dump(MOCK_PARSE_MACFILE_AWS_ROLE, default_flow_style=False))
        self.assertEquals(yaml.dump(infrastructures, default_flow_style=False), yaml.dump(MOCK_PARSE_MACFILE_AWS_INF, default_flow_style=False))

    def test_open_file_no_order(self):
        contents = maccli.helper.macfile.load_macfile("%s/aws-medium-pgbench.macfile" % self.mock_path)
        macfile_raw = maccli.dao.inheritance.resolve_inheritance(contents, [])
        root, roles, infrastructures, _, _ = maccli.service.macfile.parse_macfile(macfile_raw)
        self.assertNotEquals(yaml.dump(infrastructures, default_flow_style=False), yaml.dump(MOCK_PARSE_MACFILE_AWS_NO_ORDER_INF, default_flow_style=False))

    def test_no_roles(self):
        contents = maccli.helper.macfile.load_macfile("%s/vpc.aws.macfile" % self.mock_path)
        macfile_raw = maccli.dao.inheritance.resolve_inheritance(contents, [])
        root, roles, infrastructures, actions, resources = maccli.service.macfile.parse_macfile(macfile_raw)
        self.assertEquals(yaml.dump(root, default_flow_style=False), yaml.dump({'version': '1.0', 'name': 'demo'}, default_flow_style=False))
        self.assertEquals(yaml.dump(roles, default_flow_style=False), yaml.dump([], default_flow_style=False))
        self.assertEquals(yaml.dump(infrastructures, default_flow_style=False), yaml.dump(OrderedDict([('vpc_inf', OrderedDict([('resource', 'create_vpc')])), ('subnet_inf', OrderedDict([('resource', 'create_subnet')])), ('subnet_enable_public_ips_inf', OrderedDict([('resource', 'subnet_enable_public_ips')])), ('internet_gateway_inf', OrderedDict([('resource', 'create_internet_gateway')])), ('attach_internet_gateway_to_vpc', OrderedDict([('resource', 'attach_internet_gateway')])), ('attach_route_to_internet_gateway', OrderedDict([('resource', 'create_route')]))]), default_flow_style=False))
        self.assertEquals(yaml.dump(actions, default_flow_style=False), yaml.dump(OrderedDict([('get_route', OrderedDict([('bash', 'aws ec2 describe-route-tables --filters Name=vpc-id,Values=resource.vpc_inf.json.Vpc.VpcId --region us-east-1')]))]), default_flow_style=False))
        self.assertEquals(resources, OrderedDict([('create_vpc', OrderedDict([('create bash', 'aws ec2 create-vpc --cidr-block 10.0.0.0/16 --region us-east-1'), ('destroy bash', 'aws ec2 delete-vpc --vpc-id resource.vpc_inf.json.Vpc.VpcId --region us-east-1')])), ('create_subnet', OrderedDict([('create bash', 'aws ec2 create-subnet --vpc-id resource.vpc_inf.json.Vpc.VpcId --cidr-block 10.0.1.0/24 --region us-east-1'), ('destroy bash', 'aws ec2 delete-subnet --subnet-id resource.subnet_inf.json.Subnet.SubnetId --region us-east-1')])), ('subnet_enable_public_ips', OrderedDict([('create bash', 'aws ec2 modify-subnet-attribute --subnet-id resource.subnet_inf.json.Subnet.SubnetId --map-public-ip-on-launch --region us-east-1')])), ('create_internet_gateway', OrderedDict([('create bash', 'aws ec2 create-internet-gateway --region us-east-1'), ('destroy bash', 'aws ec2 delete-internet-gateway --internet-gateway-id resource.internet_gateway_inf.json.InternetGateway.InternetGatewayId --region us-east-1')])), ('attach_internet_gateway', OrderedDict([('create bash', 'aws ec2 attach-internet-gateway --internet-gateway-id resource.internet_gateway_inf.json.InternetGateway.InternetGatewayId --vpc-id resource.vpc_inf.json.Vpc.VpcId --region us-east-1'), ('destroy bash', 'aws ec2 detach-internet-gateway --internet-gateway-id resource.internet_gateway_inf.json.InternetGateway.InternetGatewayId --vpc-id resource.vpc_inf.json.Vpc.VpcId --region us-east-1')])), ('create_route', OrderedDict([('create bash', 'aws ec2 create-route --route-table-id action.get_route.json.RouteTables.0.RouteTableId --destination-cidr-block 0.0.0.0/0 --gateway-id resource.internet_gateway_inf.json.InternetGateway.InternetGatewayId --region us-east-1')]))]))

    def test_convert_args_to_yaml(self):
        yaml = maccli.service.macfile.convert_args_to_yaml(Mock_yaml_args())
        self.assertEqual(yaml, MOCK_MACFILE)

    def test_open_file_port(self):
        contents = maccli.helper.macfile.load_macfile("%s/aws-medium-pgbench.macfile" % self.mock_path)
        macfile_raw = maccli.dao.inheritance.resolve_inheritance(contents, [])
        root, roles, infrastructures, _, _ = maccli.service.macfile.parse_macfile(macfile_raw)
        self.assertEquals(yaml.dump(roles, default_flow_style=False), yaml.dump(MOCK_PARSE_MACFILE_AWS_ROLE, default_flow_style=False))
        self.assertEquals(yaml.dump(infrastructures, default_flow_style=False), yaml.dump(MOCK_PARSE_MACFILE_AWS_INF, default_flow_style=False))

    def test_parse_infrastructure_params(self):
        contents = maccli.helper.macfile.load_macfile("%s/infrastructure.params.macfile" % self.mock_path)
        macfile_raw = maccli.dao.inheritance.resolve_inheritance(contents, [])
        root, _, infrastructures, _, _ = maccli.service.macfile.parse_macfile(macfile_raw)
        self.assertEquals(yaml.dump(infrastructures, default_flow_style=False), yaml.dump(MOCK_PARSE_MACFILE_INFRASTURCTURE_PARAMS, default_flow_style=False))

    def test_open_file_infrastructure(self):
        contents = maccli.helper.macfile.load_macfile("%s/infrastructure.aws.macfile" % self.mock_path)
        macfile_raw = maccli.dao.inheritance.resolve_inheritance(contents, [])
        root, roles, infrastructures, actions, resources = maccli.service.macfile.parse_macfile(macfile_raw)

        self.assertEquals(yaml.dump(root, default_flow_style=False), yaml.dump(MOCK_PARSE_MACFILE_V2_EXPECTED_ROOT, default_flow_style=False))
        self.assertEquals(yaml.dump(roles, default_flow_style=False), yaml.dump(MOCK_PARSE_MACFILE_V2_EXPECTED_ROLES, default_flow_style=False))
        self.assertEquals(yaml.dump(infrastructures, default_flow_style=False), yaml.dump(MOCK_PARSE_MACFILE_V2_EXPECTED_INFRASTRUCTURES, default_flow_style=False))
        self.assertEquals(yaml.dump(actions, default_flow_style=False), yaml.dump(MOCK_PARSE_MACFILE_V2_EXPECTED_ACTIONS, default_flow_style=False))
        self.assertEquals(resources, MOCK_PARSE_MACFILE_V2_EXPECTED_RESOURCES)
        #self.assertEquals(yaml.dump(resources, default_flow_style=False), yaml.dump(MOCK_PARSE_MACFILE_V2_EXPECTED_RESOURCES, default_flow_style=False))

    def test_parse_params_valid(self):
        raw_params, found = maccli.helper.macfile.parse_params(MOCK_MACFILE_PARAMS, MACFILE_PARAMS_VALID)
        self.assertEqual(raw_params, MOCK_MACFILE_PARAMS_VALID)

    def test_parse_params_valid_2(self):
        raw_params, found = maccli.helper.macfile.parse_params(MOCK_MACFILE_PARAMS_2, MACFILE_PARAMS_EMPTY)
        self.assertEqual(raw_params, MOCK_MACFILE_PARAMS_2)

    def test_parse_params_invalid(self):
        self.assertRaises(MacParseParamException, maccli.helper.macfile.parse_params, MOCK_MACFILE_PARAMS, MACFILE_PARAMS_INVALID)

    def test_parse_params_missing(self):
        self.assertRaises(MacParseParamException, maccli.helper.macfile.parse_params, MOCK_MACFILE_PARAMS, MACFILE_PARAMS_ONE_MISSING)





import unittest

from tests.mock_data import *
import maccli.helper.network


class AuthTestCase(unittest.TestCase):
    def test_network(self):
        self.assertTrue(maccli.helper.network.is_ip_private("127.0.0.1"))
        self.assertTrue(maccli.helper.network.is_ip_private("192.168.0.1"))
        self.assertFalse(maccli.helper.network.is_ip_private("162.243.152.74"))
        self.assertTrue(maccli.helper.network.is_ip_private("10.10.10.10"))
        self.assertFalse(maccli.helper.network.is_ip_private("172.2.1.2"))
        self.assertTrue(maccli.helper.network.is_ip_private("172.16.1.2"))
        self.assertTrue(maccli.helper.network.is_ip_private("172.30.1.2"))
        self.assertTrue(maccli.helper.network.is_ip_private("172.31.1.2"))
        self.assertTrue(maccli.helper.network.is_ip_private("172.31.36.70"))
        self.assertFalse(maccli.helper.network.is_ip_private("172.32.1.2"))

    def test_local_loop(self):
        self.assertTrue(maccli.helper.network.is_local("127.0.0.1"))
        self.assertFalse(maccli.helper.network.is_local("192.168.0.1"))
        self.assertFalse(maccli.helper.network.is_local("162.243.152.74"))
        self.assertFalse(maccli.helper.network.is_local("10.10.10.10"))
        self.assertFalse(maccli.helper.network.is_local("172.2.1.2"))
        self.assertFalse(maccli.helper.network.is_local("172.16.1.2"))
        self.assertFalse(maccli.helper.network.is_local("172.30.1.2"))
        self.assertFalse(maccli.helper.network.is_local("172.31.1.2"))
        self.assertFalse(maccli.helper.network.is_local("172.32.1.2"))



# Simple cache module. Just a dictionary to save key values for this session

import maccli
import hashlib

cache = {}  # Cache dictionary


def get(key):

    if key in cache:
        value = cache[key]
        maccli.logger.debug("Getting %s from cache success: %s" % (key, value))
        return value
    else:
        maccli.logger.debug("Getting %s from cache failed" % key)
        return None


def set_value(key, value):
    maccli.logger.debug("Setting key '%s' with value %s" % (key, value))
    cache[key] = value


def hash_value(value):
    md5 = hashlib.md5()
    md5.update(value.encode())
    return md5.hexdigest()



import unittest

import mock

import maccli.service.instance
from tests.mock_data import *


DEFAULT_SERVERNAME = "servername"
DEFAULT_SESSIONID = "sessionid"

class AuthTestCase(unittest.TestCase):

    @mock.patch('maccli.dao.api_instance.get_list')
    def test_list_configurations(self, mock):
        mock.return_value = MOCK_INSTANCE_LIST_JSON
        json_response = maccli.service.instance.list_instances()
        self.assertTrue(mock.called)
        self.assertEqual(json_response, MOCK_INSTANCE_LIST_JSON)

    @mock.patch('maccli.dao.api_instance.get_list')
    def test_list_configurations_filter_ids(self, mock):
        SELECTED_INSTANCES = [{u'status': u'Ready', u'servername': u'mcp-prod-b55', u'lifespan': 0, u'ipv4': u'166.78.136.158', u'type': u'production', u'id': u'vcb553nb5p40km5tc1ofl1t0t8', u'metadata': {u'system': {u'infrastructure': {u'hardware': u'DFW/2', u'provider': u'rackspaceus', u'location': u'DFW', u'lifespan': -1, u'deployment': u'production'}, u'role': {u'environment': {u'PASSWORD': u'4br4c4d4br4yeh', u'VPN_SERVER': u'10.0.1.113', u'IP_RANGE': u'10.0.1.114-126'}, u'cookbook_tag': u'base_worker', u'block_tags': [u'meeou161t7i4uhtuj6i0utc7n']}}}}]
        mock.return_value = MOCK_INSTANCE_LARGE_LIST_JSON
        response = maccli.service.instance.list_instances(name_or_ids="vcb553nb5p40km5tc1ofl1t0t8")
        self.assertTrue(mock.called)
        self.assertEqual(response, SELECTED_INSTANCES)

    @mock.patch('maccli.dao.api_instance.get_list')
    def test_list_configurations_filter_ids(self, mock):
        SELECTED_INSTANCES = [{u'status': u'Ready', u'servername': u'mcp-prod-b55', u'lifespan': 0, u'ipv4': u'166.78.136.158', u'type': u'production', u'id': u'vcb553nb5p40km5tc1ofl1t0t8', u'metadata': {u'system': {u'infrastructure': {u'hardware': u'DFW/2', u'provider': u'rackspaceus', u'location': u'DFW', u'lifespan': -1, u'deployment': u'production'}, u'role': {u'environment': {u'PASSWORD': u'4br4c4d4br4yeh', u'VPN_SERVER': u'10.0.1.113', u'IP_RANGE': u'10.0.1.114-126'}, u'cookbook_tag': u'base_worker', u'block_tags': [u'meeou161t7i4uhtuj6i0utc7n']}}}}, {u'status': u'Ready', u'servername': u'mcp-prod-d8b', u'lifespan': 0, u'ipv4': u'162.209.99.168', u'type': u'production', u'id': u'2sh6jr9eb3gqgls3gasi7nb9q2', u'metadata': {u'system': {u'infrastructure': {u'hardware': u'IAD/2', u'provider': u'rackspaceus', u'location': u'IAD', u'lifespan': -1, u'deployment': u'production'}, u'role': {u'environment': {u'PASSWORD': u'4br4c4d4br4yeh', u'VPN_SERVER': u'10.0.1.145', u'IP_RANGE': u'10.0.1.146-158'}, u'cookbook_tag': u'base_worker', u'block_tags': [u'meeou161t7i4uhtuj6i0utc7n']}}}}]
        mock.return_value = MOCK_INSTANCE_LARGE_LIST_JSON
        response = maccli.service.instance.list_instances(name_or_ids=["vcb553nb5p40km5tc1ofl1t0t8", "2sh6jr9eb3gqgls3gasi7nb9q2"])
        self.assertTrue(mock.called)
        self.assertEqual(response, SELECTED_INSTANCES)


    @mock.patch('maccli.dao.api_instance.get_list')
    def test_list_by_infrastructure(self, mock):
        mock.return_value = MOCK_INSTANCE_LIST_INFRASTRUCTURE_RAW
        instances = maccli.service.instance.list_by_infrastructure('testing', '1.0')
        self.assertTrue(mock.called)
        self.assertEqual(instances, MOCK_INSTANCE_LIST_INFRASTRUCTURE_CLEAN)





# @mock.patch('os.system')
    # @mock.patch('maccli.dao.api_instance.credentials')
    # def test_ssh_instance_privateKey(self, mock_credentials, mock_os):
    #     mock_credentials.return_value = MOCK_INSTANCE_CREDENTIALS_PRIVKEY_JSON
    #     maccli.service.instance.ssh_instance(DEFAULT_SERVERNAME, DEFAULT_SESSIONID)
    #     mock_os.assert_called_once_with("ssh root@104.236.164.139 -i /tmp/tmpfVO0A8")

    # @mock.patch('pexpect.spawn')
    # @mock.patch('dao.api_instance.credentials')
    # def test_ssh_instance_privateKey(self, mock_credentials, mock_os):
    #     mock_credentials.return_value = MOCK_INSTANCE_CREDENTIALS_PASS_JSON
    #     maccli.service.instance.ssh_instance(DEFAULT_SERVERNAME, DEFAULT_SESSIONID)







from io import StringIO
import unittest

import mock
import sys

import maccli.view.view_infrastructure
from tests.mock_data import *

INFRASTRUCTURES = OrderedDict([('app_inf', OrderedDict([('name', 'app'), ('provider', 'amazon'), ('location', 'us-east-1'), ('hardware', 't1.micro'), ('role', 'app')])), ('build_lb_inf', OrderedDict([('resource', 'build_lb')])), ('register_lb_inf', OrderedDict([('resource', 'register_lb')]))])
RESOURCES_PROCESSED = [{'build_lb_inf': {'rc': 0, 'stderr': '', 'stdout': '{\n    "DNSName": "my-load-balancer-1110685180.us-east-1.elb.amazonaws.com"\n}\n'}}, {'register_lb_inf': {'rc': 0, 'stderr': '', 'stdout': '{\n    "Instances": [\n        {\n            "InstanceId": "i-d3911600"\n        }\n    ]\n}\n'}}]


class InfrastructureViewTestCase(unittest.TestCase):
    def setUp(self):
        self.stderr = sys.stderr
        self.stdout = sys.stdout
        sys.stderr = self.buf = StringIO()
        sys.stdout = self.bufout = StringIO()
        # pass

    def tearDown(self):
        sys.stderr = self.stderr
        sys.stdout = self.stdout
        # pass

    def test_show_resources_status(self):
        EXPECTED =  "+-----------------+---------+\n"\
                    "| Resource name   | Status  |\n"\
                    "+-----------------+---------+\n" \
                    "| app_inf         | Pending |\n" \
                    "| build_lb_inf    | OK      |\n"\
                    "| register_lb_inf | OK      |\n"\
                    "+-----------------+---------+\n"
        maccli.view.view_infrastructure.show_infrastructure_resources(INFRASTRUCTURES, RESOURCES_PROCESSED)
        actual = self.bufout.getvalue()
        self.assertEqual(actual, EXPECTED)

    def test_show_resources_in_infrastructure_status(self):
        INFRASTRUCTURES_RESOURCES = [{u'cloudServers': [], u'name': u'demo', u'resources': [{u'create': {u'cmd': u'aws ec2 create-vpc --cidr-block 10.0.0.0/16 --region us-east-1', u'rc': 0, u'stderr': u'', u'stdout': u'{\n    "Vpc": {\n        "InstanceTenancy": "default", \n        "State": "pending", \n        "VpcId": "vpc-b78c48d3", \n        "CidrBlock": "10.0.0.0/16", \n        "DhcpOptionsId": "dopt-838273e6"\n    }\n}\n'}, u'metadata': {u'infrastructure': {u'macfile_infrastructure_name': u'vpc_inf', u'version': u'1.0', u'name': u'demo', u'macfile_resource_name': u'create_vpc'}}, u'cmdDestroy': u'aws ec2 delete-vpc --vpc-id resource.vpc_inf.json.Vpc.VpcId', u'name': u'vpc_inf', u'cmdCreate': u'aws ec2 create-vpc --cidr-block 10.0.0.0/16 --region us-east-1'}], u'versions': [u'1.0']}]

        EXPECTED =  "+---------------------+---------+---------------+---------------+--------+\n"\
                    "| Infrastructure name | Version | Resource type | Resource name | Status |\n"\
                    "+---------------------+---------+---------------+---------------+--------+\n"\
                    "| demo                | 1.0     | create_vpc    | vpc_inf       | Ready  |\n"\
                    "+---------------------+---------+---------------+---------------+--------+\n"

        maccli.view.view_infrastructure.show_resources_in_infrastructure(INFRASTRUCTURES_RESOURCES)
        actual = self.bufout.getvalue()
        self.assertEqual(actual, EXPECTED)



import re
import argparse
from maccli.config import MACFILE_ON_FAILURE_DESTROY_ALL, MACFILE_ON_FAILURE_DESTROY_OTHERS


def add_login_parser(subparsers):
    subparsers.add_parser('login', help='Login into Manageacloud.com', description='Login into Manageacloud.com')


def add_instance_parser(subparsers):
    instance_parser = subparsers.add_parser('instance', aliases='ins',
                                            help='Instance related operations',
                                            description='Server instances operations')

    instance_subparser = instance_parser.add_subparsers(title='mac instance commands', dest='subcmd')

    # list instance
    list_parser = instance_subparser.add_parser('list',
                                                help='List testing and production server instances',
                                                description='List testing and production server instances available in your account')

    # create instance
    create_parser = instance_subparser.add_parser('create', help='Create a new instance',
                                                  description='Creates a new instance in the cloud. You need to choose the '
                                                              'configuration that you want to apply. Please note that '
                                                              'you can create testing servers, which have a limited lifespan, '
                                                              'and production servers, '
                                                              'that must be destroyed manually.')

    create_parser.add_argument('-c', '--configuration', help='Configuration tag')

    create_parser.add_argument('-b', '--bootstrap', help='Bash commands, script file or URL')

    create_parser.add_argument('-l', '--location',
                               help='Location name. If no provided, the list of available locations will be displayed.')

    create_parser.add_argument('-d', '--deployment', default="testing", choices=["testing", "production"],
                               help="Choose the type of server. Testing servers will has a limited lifespan (default is 'testing')")

    #create_parser.add_argument('-b', '--branch', default="master", choices=["development", "master"], help=argparse.SUPPRESS)
    #                          # help="Select the branch. This only applies if the provider is 'manageacloud (default is 'master')")

    create_parser.add_argument('-p', '--provider', default="default",
                               choices=["default", "rackspaceus", "rackspaceuk", "amazon", "digitalocean", "gce"],
                               help="Select the public cloud provider. (default is 'default')")

    create_parser.add_argument('-n', '--name',
                               help='Server name (default will be a random name)')

    create_parser.add_argument('-r', '--release', default="any",
                               choices=["any", "ubuntu", "ubuntu:trusty", "ubuntu:utopic", "ubuntu:vivid",
                                        "centos", "centos:6.5", "centos:7",
                                        "debian", "debian:wheezy", "debian:jessie",
                                        "amazon"],
                               help="Choose the distribution (default is 'any', which is the "
                                    "best match for 'configuration' parameter). Required for 'bootstrap' parameter.")

    create_parser.add_argument('-rv', '--release-version',
                               choices=["wheezy", "jessie", "trusty", "utopic", "vivid", "6.5", "7"],
                               help="Distribution version: Debian Wheezy or Jessie, Ubuntu Trusty, Utopic or Vivid, CentOS 6.5 or 7.")

    create_parser.add_argument('-hw', '--hardware',
                               help="Choose the hardware settings. It only applies if parameter 'deployment' is 'production'. "
                                    "If this parameter is not set, the list of the available hardware will be displayed.")

    create_parser.add_argument('-t', '--lifespan', type=int,
                               help="If deployment is 'development' choose the lifespan of the server. "
                                    "In minutes (default 90)")

    create_parser.add_argument('-e', '--environment', nargs='*', type=validate_environment,
                               help="Format KEY=VALUE. The environment variables will be available"
                                    " in the bootstrap bash script that applies the changes.")

    create_parser.add_argument('-hd', nargs='*', type=validate_hd,
                               help="For provider amazon: Format NAME:SIZE[:TYPE[:VALUE]] "
                                    "where NAME is the name of the device (example '/dev/sdb'), "
                                    "SIZE is a number in gigabytes, TYPE is the volume type (io1, gp2 or standard)"
                                    " and VALUE is the IOPS required if volume type is io1. \n"
                                    "For provider gce: NAME:SIZE:TYPE. NAME is the name of the device "
                                    "('testing' will be '/dev/disk/by-id/google-testing'), SIZE is the size "
                                    "in gigabytes and TYPE is 'ssd' or 'standard'. For rackspace: Format "
                                    "NAME:SIZE[:TYPE] where NAME is the name of the device (example '/dev/sdb')"
                                    " SIZE is the size in gb (min 100) and TYPE is 'SSD' or 'SATA'")

    create_parser.add_argument('--port', type=validate_port,
                               help="List of ports that are opened. "
                                    "The port 22 (SSH) must be in the list. "
                                    "Example: 22,80,8020")

    create_parser.add_argument('--net',
                               help="Network related property. For provider 'amazon', VPC subnet id. ")


    create_parser.add_argument('-y', '--yaml', action='store_true', default=False,
                               help="Prints the equivalent command in Macfile and exits.")

    # update instance
    update_parser = instance_subparser.add_parser('update',
                                                   help='Run the server configuration',
                                                   description='Run the server configuration')
    update_parser.add_argument('id', nargs='*',
                                help="Server ID or server name. It also accepts several values. If value is 'all' it applies to all servers.")

    update_parser.add_argument('-c', '--configuration', help='Configuration tag')

    update_parser.add_argument('-b', '--bootstrap', help='Bash commands, script file or URL')

    # destroy instance
    destroy_parser = instance_subparser.add_parser('destroy',
                                                   help='Destroy an existing instance',
                                                   description='Destroy an existing instance')

    destroy_parser.add_argument('id', nargs='*',
                                help='Server ID or server name')

    # ssh instance
    ssh_parser = instance_subparser.add_parser('ssh',
                                       help='Connect via SSH',
                                       description='Connect via SSH to the server')

    ssh_parser.add_argument('id', nargs='*',
                            help="Server ID or server name. It also accepts several values. If value is 'all' it applies to all servers.")

    ssh_parser.add_argument('-c', '--command', nargs=argparse.REMAINDER, help='Run a command and exit')

    # facts instance
    facts_parser = instance_subparser.add_parser('facts',
                                                 help='Retrieves facts about an instance',
                                                 description='Retrieves facts about  an instance')
    facts_parser.add_argument('id', help='Server ID or server name')

    # logs
    logs_parser = instance_subparser.add_parser('log',
                                                help='Show instance logs',
                                                description='Show server output when creating and applying '
                                                            'the configuration to an instance')

    logs_parser.add_argument('id', help='Server ID or server name')

    logs_parser.add_argument('-f', '--follow', action='store_true', default=False,
                             help="Output appends data as server is configured.")

    # lifespan operations
    lifespan_parser = instance_subparser.add_parser('lifespan',
                                                    help='Manipulate testing instance\'s lifespan',
                                                    description='Add or remove testing instance lifespan')
    lifespan_parser.add_argument('id', help='Server ID or server name')
    lifespan_parser.add_argument('amount', type=int, help='New server lifespan in minutes')


def add_resource_parser(subparsers):
    inf_parser = subparsers.add_parser('resource',
                                       help='Resource operations',
                                       description='Resource related operations')

    inf_subparser = inf_parser.add_subparsers(title='resource operations', dest='subcmd')

    # provider save credentials
    #stderr_parser = inf_subparser.add_parser('get_stderr', help='Get error output from resource',
    #                                              description='Gets the error output from a resource ')
    stdout_parser = inf_subparser.add_parser('get_stdout', help='Get output from resource',
                                            description='Get output from resource. Use the command "mac infrastructure items" to get a list of available resources.')

    stdout_parser.add_argument('infrastructure_name', help="Infrastructure name")
    stdout_parser.add_argument('infrastructure_version', help="Infrastructure version")
    stdout_parser.add_argument('resource_name', help="Resource name ")
    stdout_parser.add_argument('key', nargs='?', help='Parses the output using "json" or "text.regex". Example: json.My_Key, text.regex(.*)')


def add_provider_parser(subparsers):
    inf_parser = subparsers.add_parser('provider',
                                       help='Provider operations',
                                       description='Provider related operations')

    inf_subparser = inf_parser.add_subparsers(title='provider operations', dest='subcmd')

    # provider save credentials
    credentials_parser = inf_subparser.add_parser('credential', help='Save cloud supplier credentials',
                                              description='Allows to activate a new cloud supplier. '
                                                          'Those credentials are used if you want to create new instances (or "roles" in the "macfiles"). '
                                                          'For more information to provide the correct credentials please go to '
                                                          'https://manageacloud.com/article/orchestration/cli/provider/credential')
    credentials_parser.add_argument('provider',
                               choices=["rackspaceus", "rackspaceuk", "amazon", "digitalocean", "gce"],
                               help="Select the public cloud provider.")
    credentials_parser.add_argument('clientid', help='User identification: clientId, username, etc')
    credentials_parser.add_argument('key', help='Secret information: password, key, pem, etc. If argument is an existing file path the contents of the file will be used.')
    credentials_parser.add_argument('-f', '--force-file', action='store_true', help='Force "key" as file path.')


def add_infrastructure_parser(subparsers):
    """ infrastructure parser"""
    inf_parser = subparsers.add_parser('infrastructure', aliases=('infra'),
                                       help='Infrastructure operations',
                                       description='Infrastructure related operations')

    inf_subparser = inf_parser.add_subparsers(title='infrastructure operations', dest='subcmd')

    # infrastructure macfile
    macfile_parser = inf_subparser.add_parser('macfile', help='Create an infrastructure',
                                              description='Create an infrastructure loading a macfile')

    # get file path
    macfile_parser.add_argument('file', nargs=1, help='Path or URL to Macfile')
    macfile_parser.add_argument('--resume', action='store_true', help="Resume infrastructure creation")
    macfile_parser.add_argument('-p', '--param', nargs='*', help="Add parameters to be replaced in the macfile")
    macfile_parser.add_argument('--on_failure', choices=[MACFILE_ON_FAILURE_DESTROY_ALL, MACFILE_ON_FAILURE_DESTROY_OTHERS],
                                help="Action to be taken a server fails. 'destroy_all' "
                                     "will completely remove every server created. "
                                     "To allow debugging, 'destroy_others' will destroy "
                                     "all the servers that did not fail (this allows to debug)")

    # infrastructure list
    inf_subparser.add_parser('list', help='List all infrastructure and versions available')

    # infrastructure instance
    instance_parser = inf_subparser.add_parser('items', help='List instances and resources available in a infrastructure')
    instance_parser.add_argument('-v', '--version', help='Filter by infrastructure version')
    instance_parser.add_argument('-n', '--name', help='Filter by infrastructure name')

    # infrastructure destroy
    instance_parser = inf_subparser.add_parser('destroy', help='Destroy instances and resources')
    instance_parser.add_argument('name', help='Infrastructure name')
    instance_parser.add_argument('version', help='Infrastructure version')

    # infrastructure update
    instance_parser = inf_subparser.add_parser('update', help='Update instance configuration')
    instance_parser.add_argument('name', help='Infrastructure name')
    instance_parser.add_argument('version', help='Infrastructure version')
    instance_parser.add_argument('-c', '--configuration', help='Configuration tag')

    # lifespan
    lifespan_parser = inf_subparser.add_parser('lifespan', help='Manipulate testing instance\'s lifespan')
    lifespan_parser.add_argument('amount', type=int, help='New server lifespan in minutes')
    lifespan_parser.add_argument('-v', '--version', help='Filter by infrastructure version')
    lifespan_parser.add_argument('-n', '--name', help='Filter by infrastructure name')

    # ssh add
    lifespan_parser = inf_subparser.add_parser('sshkey', help='Gather ssh public keys')
    lifespan_parser.add_argument('-v', '--version', help='Filter by infrastructure version')
    lifespan_parser.add_argument('-n', '--name', help='Filter by infrastructure name')
    lifespan_parser.add_argument('-k', '--known_host', action='store_true', help='Add new ssh keys to known_host file')


def add_configuration_parser(subparsers):
    configuration_parser = subparsers.add_parser('configuration', help='Server configuration related operations',
                                                 description='Search public and private configurations')

    configuration_subparser = configuration_parser.add_subparsers(title='mac configuration commands', dest='subcmd')

    # list instance
    list_parser = configuration_subparser.add_parser('list',
                                                     help='List your server configurations',
                                                     description='List the server configurations available in your account')

    search_parser = configuration_subparser.add_parser('search',
                                                       help='Search public server configurations',
                                                       description='Search public server configurations available in Manageacloud.com')

    search_parser.add_argument('-k', '--keyword',
                               help='Keywords', nargs='*')

    search_parser.add_argument('-u', '--url',
                               help='Show Urls', action='store_true', default=False)


def validate_environment(input):
    """
        Checks that the input parameter has the format
        KEY=VALUE
    """
    a = re.compile("^[A-Z0-9\_\-]+\=(.|\n)+$", re.IGNORECASE)
    match = a.match(input)
    if not match:
        msg = "'%s' environment contains invalid characters or the format KEY=VALUE is not correct" % input
        raise argparse.ArgumentTypeError(msg)
    key, value = input.split("=", 1)
    to_return = {key: value}
    return to_return


def validate_hd(input):
    """
        checks that the input
        /dev/name:SIZE[:type[:value]] is correct./
    """
    a = re.compile("^[/a-zA-Z1-9]+:[0-9]+(:[a-zA-Z0-9]+(:[a-zA-Z0-9]+)?)?$", re.IGNORECASE)
    match = a.match(input)
    if not match:
        msg = "'%s' hard-disk is invalid. Correct value NAME:SIZE<:TYPE<:VALUE>> name are letters and " \
              "number and SIZE is a number that represents the gigabytes." % input
        raise argparse.ArgumentTypeError(msg)
    key, value = input.split(":", 1)
    to_return = {key: value}
    return to_return


def validate_port(port_input):
    """
        checks that the input 22,80,2020 is correct
    """
    ports_raw = port_input.split(',')
    ports = []
    ssh_port = False
    for port_raw in ports_raw:
        try:
            port = int(port_raw)
            if port == 22:
                ssh_port = True
            if port < 1 or port > 65535:
                msg = "'%s' doesn't look valid. The value %s should be between 1 and 65535" % (port_raw, port_input)
                raise argparse.ArgumentTypeError(msg)

        except ValueError:
            msg = "'%s' doesn't look valid. The value %s is not an integer" % (port_raw, port_input)
            raise argparse.ArgumentTypeError(msg)
        ports.append(port)

    if not ssh_port:
        msg = "'%s' doesn't look valid. You need to allow access to the port 22" % port_input
        raise argparse.ArgumentTypeError(msg)

    return ports



from collections import OrderedDict
import yaml, yaml.resolver

class UnsortableList(list):
    def sort(self, *args, **kwargs):
        pass

class UnsortableOrderedDict(OrderedDict):
    def items(self, *args, **kwargs):
        return UnsortableList(OrderedDict.items(self, *args, **kwargs))


def ordered_load(stream, Loader=yaml.Loader, object_pairs_hook=OrderedDict):
    class OrderedLoader(Loader):
        pass

    def construct_mapping(loader, node):
        #loader.flatten_mapping(node)
        return object_pairs_hook(loader.construct_pairs(node))
    OrderedLoader.add_constructor(
        yaml.resolver.BaseResolver.DEFAULT_MAPPING_TAG,
        construct_mapping)
    return yaml.load(stream, OrderedLoader)


from io import StringIO
import unittest

import mock
import sys

import maccli.mac_cli
from tests.mock_data import *


class MacTestCase(unittest.TestCase):
    def setUp(self):
        maccli.user = MOCK_USER
        maccli.apikey = MOCK_APIKEY

    @mock.patch('maccli.command_cli.login')
    def test_dispatch_cmds_login(self, mock):
        args = Mock_args('login', None)
        maccli.mac_cli.dispatch_cmds(args)
        self.assertTrue(mock.called)

    @mock.patch('maccli.command_cli.instance_create')
    def test_dispatch_cmds_create(self, mock):
        args = MockInstanceCreate_args('instance', 'create')
        maccli.mac_cli.dispatch_cmds(args)
        self.assertTrue(mock.called)

    @mock.patch('maccli.command_cli.instance_destroy_help')
    def test_dispatch_cmds_destroy_help(self, mock):
        self.stderr = sys.stderr
        sys.stderr = StringIO()
        args = MockInstanceDestroy_args('instance', 'destroy', None, None)
        maccli.mac_cli.dispatch_cmds(args)
        self.assertTrue(mock.called)

    @mock.patch('maccli.command_cli.instance_list')
    def test_dispatch_cmds_list(self, mock):
        args = Mock_args('instance', 'list')
        maccli.mac_cli.dispatch_cmds(args)
        self.assertTrue(mock.called)

    @mock.patch('maccli.command_cli.instance_ssh_help')
    def test_dispatch_cmds_ssh_help(self, mock):
        self.stderr = sys.stderr
        sys.stderr = StringIO()
        args = MockInstanceSSH_args('instance', 'ssh', None, None)
        maccli.mac_cli.dispatch_cmds(args)
        self.assertTrue(mock.called)

    @mock.patch('maccli.command_cli.instance_ssh')
    def test_dispatch_cmds_ssh_help_1(self, mock):
        args = MockInstanceSSH_args('instance', 'ssh', 'nameorid', None)
        maccli.mac_cli.dispatch_cmds(args)
        self.assertTrue(mock.called)

    @mock.patch('maccli.command_cli.configuration_list')
    def test_dispatch_cmds_configuration_list(self, mock):
        args = Mock_args('configuration', 'list')
        maccli.mac_cli.dispatch_cmds(args)
        self.assertTrue(mock.called)

    @mock.patch('maccli.command_cli.configuration_search')
    def test_dispatch_cmds_configuration_Search(self, mock):
        args = MockConfiguration_Args('configuration', 'search', None, None)
        maccli.mac_cli.dispatch_cmds(args)
        self.assertTrue(mock.called)



import maccli.view.view_generic
from prettytable import PrettyTable


def show_resources(resources):

    if len(resources):
        for resource in resources:
            for key in resource.keys():
                r = resource[key]
                maccli.view.view_generic.header("Resource %s" % key, "-")
                if r['rc']:
                    maccli.view.view_generic.show("Process failed. Return code: %s. Output:" % r['rc'])
                    maccli.view.view_generic.show(r['stderr'])
                    if r['stdout'] is not None and r['stdout'].strip():
                        maccli.view.view_generic.show("Standard output not empty:")
                        maccli.view.view_generic.show(r['stdout'])
                else:
                    maccli.view.view_generic.show("Execution successful. Output:")
                    maccli.view.view_generic.show(r['stdout'])
                    if r['stderr'] is not None and r['stderr'].strip():
                        maccli.view.view_generic.show("Standard error not empty:")
                        maccli.view.view_generic.show(r['stderr'])

    else:
        print("There are no resources available")




import os
import tempfile
import time
import urllib
import urllib.parse

import pexpect

import maccli.dao.api_instance
import maccli.helper.cmd
import maccli.helper.simplecache
import maccli.helper.metadata
from maccli.helper.exception import InstanceDoesNotExistException, InstanceNotReadyException


def list_instances(name_or_ids=None):
    """
        List available instances in the account
    """
    instances = []
    instances_raw = maccli.dao.api_instance.get_list()
    if name_or_ids is not None:

        # if name_or_ids is string, convert to list
        if isinstance(name_or_ids, str):
            name_or_ids = [name_or_ids]

        for instance_raw in instances_raw:
            if instance_raw['servername'] in name_or_ids or instance_raw['id'] in name_or_ids:
                instances.append(instance_raw)
    else:
        instances = instances_raw

    return instances


def list_by_infrastructure(name, version):
    instances = maccli.dao.api_instance.get_list()
    filtered_instances = []
    for instance in instances:
        if 'metadata' in instance and 'infrastructure' in instance['metadata'] and 'name' in instance['metadata'][
            'infrastructure']:
            infrastructure_name = instance['metadata']['infrastructure']['name']
        else:
            infrastructure_name = ""

        if 'metadata' in instance and 'infrastructure' in instance['metadata'] and 'version' in instance['metadata'][
            'infrastructure']:
            infrastructure_version = instance['metadata']['infrastructure']['version']
        else:
            infrastructure_version = ""

        if infrastructure_name == name and infrastructure_version == version:
            filtered_instances.append(instance)

    return filtered_instances


def ssh_command_instance(instance_id, cmd):
    rc, stdout, stderr = -1, "", ""
    cache_hash = maccli.helper.simplecache.hash_value(cmd)
    cache_key = 'ssh_%s_%s' % (instance_id, cache_hash)
    cached_value = maccli.helper.simplecache.get(cache_key)  # read from cache

    if cached_value is not None:
        rc = cached_value['rc']
        stdout = cached_value['stdout']
        stderr = cached_value['stderr']
    else:

        instance = maccli.dao.api_instance.credentials(instance_id)

        # strict host check
        ssh_params = ""
        if maccli.disable_strict_host_check:
            maccli.logger.debug("SSH String Host Checking disabled")
            ssh_params = "-o 'StrictHostKeyChecking no'"

        if instance is None:
            raise InstanceDoesNotExistException(instance_id)

        if not (instance['privateKey'] or instance['password']):
            raise InstanceNotReadyException(instance_id)

        if instance is not None and (instance['privateKey'] or instance['password']):
            if instance['privateKey']:
                fd, path = tempfile.mkstemp()
                try:
                    with open(path, "wb") as f:
                        f.write(bytes(instance['privateKey'], encoding='utf8'))
                        f.close()

                    # openssh 7.6, it defaults to a new more secure format.
                    command = "ssh-keygen -f %s -p -N ''" % f.name
                    maccli.helper.cmd.run(command)

                    command = "ssh %s %s@%s -i %s %s" % (ssh_params, instance['user'], instance['ip'], f.name, cmd)

                    rc, stdout, stderr = maccli.helper.cmd.run(command)

                finally:
                    os.close(fd)
                    os.remove(path)

            else:
                """ Authentication with password """
                command = "ssh %s %s@%s %s" % (ssh_params, instance['user'], instance['ip'], cmd)
                child = pexpect.spawn(command)
                (rows, cols) = gettermsize()
                child.setwinsize(rows, cols)  # set the child to the size of the user's term
                i = child.expect(['.* password:', "yes/no", '(.*)'], timeout=60)
                if i == 0:
                    child.sendline(instance['password'])
                    child.expect(pexpect.EOF, timeout=120)
                elif i == 1:
                    child.sendline("yes")
                    child.expect('.* password:', timeout=60)
                    child.sendline(instance['password'])
                    child.expect(pexpect.EOF, timeout=120)
                elif i == 2:
                    child.expect(pexpect.EOF, timeout=120)

                output = child.before

                while child.isalive():
                    time.sleep(0.1)

                rc = child.exitstatus

                # HACK: we do not really capture stderr
                if rc:
                    stdout = ""
                    stderr = output
                else:
                    stdout = output
                    stderr = ""

        # save cache
        if not rc:
            cached_value = {
                'rc': rc,
                'stdout': stdout,
                'stderr': stderr
            }
            maccli.helper.simplecache.set_value(cache_key, cached_value)

    return rc, stdout, stderr


def ssh_interactive_instance(instance_id):
    """
        ssh to an existing instance for an interactive session
    """
    stdout = None
    instance = maccli.dao.api_instance.credentials(instance_id)

    # strict host check
    ssh_params = ""
    maccli.logger.debug("maccli.strict_host_check: " + str(maccli.disable_strict_host_check))
    if maccli.disable_strict_host_check:
        maccli.logger.debug("SSH String Host Checking disabled")
        ssh_params = "-o 'StrictHostKeyChecking no'"

    if instance is not None:

        if instance['privateKey']:
            """ Authentication with private key """
            fd, path = tempfile.mkstemp()
            try:
                with open(path, "wb") as f:
                    f.write(bytes(instance['privateKey'], encoding='utf8'))
                    f.close()

                command = "ssh %s %s@%s -i %s " % (ssh_params, instance['user'], instance['ip'], f.name)
                os.system(command)

            finally:
                os.close(fd)
                os.remove(path)

        else:
            """ Authentication with password """
            command = "ssh %s %s@%s" % (ssh_params, instance['user'], instance['ip'])
            child = pexpect.spawn(command)
            (rows, cols) = gettermsize()
            child.setwinsize(rows, cols)  # set the child to the size of the user's term
            i = child.expect(['.* password:', "yes/no", '[#\$] '], timeout=60)
            if i == 0:
                child.sendline(instance['password'])
                child.interact()
            elif i == 1:
                child.sendline("yes")
                child.expect('.* password:', timeout=60)
                child.sendline(instance['password'])
                child.interact()
            elif i == 2:
                child.sendline("\n")
                child.interact()

    return stdout


def gettermsize():
    """ horrible non-portable hack to get the terminal size to transmit
        to the child process spawned by pexpect """
    (rows, cols) = os.popen("stty size").read().split()  # works on Mac OS X, YMMV
    rows = int(rows)
    cols = int(cols)
    return rows, cols


def create_instance(cookbook_tag, bootstrap, deployment, location, servername, provider, release, release_version,
                    branch, hardware, lifespan,
                    environments, hd, port, net, metadata=None, applyChanges=True):
    """
        List available instances in the account
    """
    return maccli.dao.api_instance.create(cookbook_tag, bootstrap, deployment, location, servername, provider, release,
                                          release_version,
                                          branch, hardware, lifespan, environments, hd, port, net, metadata,
                                          applyChanges)


def destroy_instance(instanceid):
    """

    Destroy the server

    :param servername:
    :return:
    """
    return maccli.dao.api_instance.destroy(instanceid)


def credentials(servername, session_id):
    """

    Gets the server credentials: public ip, username, password and private key

    :param instance_id;
    :return:
    """
    return maccli.dao.api_instance.credentials(servername, session_id)


def facts(instance_id):
    """

    Returns facts about the system

    :param instance_id;
    :return:
    """
    return maccli.dao.api_instance.facts(instance_id)


def log(instance_id, follow):
    """

    Returns server logs

    :param instance_id;
    :return:
    """
    if follow:
        logs = maccli.dao.api_instance.log_follow(instance_id)
    else:
        logs = maccli.dao.api_instance.log(instance_id)

    return logs


def lifespan(instance_id, amount):
    """

    Set new instance lifespan

    :param instance_id;
    :return:
    """
    return maccli.dao.api_instance.update(instance_id, amount)


def update_configuration(cookbook_tag, bootstrap, instance_id, new_metadata=None):
    """
    Update server configuration with given cookbook

    :param cookbook_tag:
    :param instance_id:
    :return:
    """

    return maccli.dao.api_instance.update_configuration(cookbook_tag, bootstrap, instance_id, new_metadata)


def create_instances_for_role(root, infrastructure, roles, infrastructure_key, quiet):
    """ Create all the instances for the given tier """

    maccli.logger.debug("Processing infrastructure %s" % infrastructure_key)

    roles_created = {}
    maccli.logger.debug("Type role")
    infrastructure_role = infrastructure['role']

    role_raw = roles[infrastructure_role]["instance create"]
    metadata = maccli.helper.metadata.metadata_instance(root, infrastructure_key, infrastructure_role, role_raw,
                                                        infrastructure)
    instances = create_tier(role_raw, infrastructure, metadata, quiet)
    roles_created[infrastructure_role] = instances

    return roles_created


def create_tier(role, infrastructure, metadata, quiet):
    """
        Creates the instances that represents a role in a given infrastructure

    :param role:
    :param infrastructure:
    :return:
    """

    lifespan = None
    try:
        lifespan = infrastructure['lifespan']
    except KeyError:
        pass

    hardware = ""
    try:
        hardware = infrastructure["hardware"]
    except KeyError:
        pass

    environment = maccli.helper.metadata.get_environment(role, infrastructure)

    hd = None
    try:
        hd = role["hd"]
    except KeyError:
        pass

    configuration = None
    try:
        configuration = role["configuration"]
    except KeyError:
        pass

    bootstrap = None
    try:
        bootstrap = role["bootstrap bash"]
    except KeyError:
        pass

    port = None
    try:
        port = infrastructure["port"]
    except KeyError:
        pass

    net = None
    try:
        net = infrastructure["net"]
    except KeyError:
        pass

    deployment = None
    try:
        deployment = infrastructure["deployment"]
    except KeyError:
        pass

    release = None
    try:
        release = infrastructure["release"]
    except KeyError:
        pass

    release_version = None
    try:
        release_version = infrastructure["release_version"]
    except KeyError:
        pass

    branch = None
    try:
        branch = infrastructure["branch"]
    except KeyError:
        pass

    provider = None
    try:
        provider = infrastructure["provider"]
    except KeyError:
        pass

    instances = []
    amount = 1
    if 'amount' in infrastructure:
        amount = infrastructure['amount']

    for x in range(0, amount):
        instance = maccli.dao.api_instance.create(configuration, bootstrap, deployment,
                                                  infrastructure["location"], infrastructure["name"],
                                                  provider, release, release_version, branch, hardware, lifespan,
                                                  environment, hd, port, net, metadata, False)
        instances.append(instance)
        maccli.logger.info("Creating instance '%s'" % (instance['id']))

    return instances



import os
import unittest

import mock
from argparse import ArgumentTypeError

from tests.mock_data import *
import maccli.parser_cli
import maccli.dao.inheritance


class ServiceInheritanceTestCase(unittest.TestCase):

    def setUp(self):
        if os.path.exists(os.getcwd() + "/mock"):
            maccli.pwd = os.getcwd() + "/mock"
        else:
            maccli.pwd = os.getcwd() + "/tests/mock"

    def test_basic_inheritance(self):
        expected = OrderedDict([('mac', '1.0.0'), ('description', 'Scaled and Load-Balanced Application'), ('name', 'demo'), ('version', '1.0'), ('roles', OrderedDict([('app', OrderedDict([('instance create', OrderedDict([('bootstrap bash', 'sudo apt-get update\nsudo apt-get install nginx -y\n'), ('environment', [OrderedDict([('DB_IP', '127.0.0.1')]), OrderedDict([('APP_BRANCH', 'master')])])]))]))])), ('infrastructures', OrderedDict([('image_base_inf', OrderedDict([('name', 'app'), ('provider', 'amazon'), ('location', 'us-east-1'), ('hardware', 't1.micro'), ('role', 'app'), ('release', 'ubuntu'), ('amount', 1)])), ('build_lb_inf', OrderedDict([('resource', 'build_lb')])), ('create_image_inf', OrderedDict([('resource', 'create_image')])), ('create_launch_configuration_inf', OrderedDict([('resource', 'create_launch_configuration')])), ('create_autoscale_group_inf', OrderedDict([('resource', 'create_autoscale_group')]))])), ('actions', {'get_id': OrderedDict([('ssh', 'wget -q -O - http://169.254.169.254/latest/meta-data/instance-id')]), 'get_security_groups': OrderedDict([('ssh', 'wget -q -O - http://169.254.169.254/latest/meta-data/security-groups')]), 'get_availability_zone': OrderedDict([('ssh', 'wget -q -O - http://169.254.169.254/latest/meta-data/placement/availability-zone')])}), ('resources', {'build_lb': OrderedDict([('create bash', 'aws elb create-load-balancer --load-balancer-name my-load-balancer --listeners Protocol=HTTP,LoadBalancerPort=80,InstanceProtocol=HTTP,InstancePort=80 --region infrastructure.image_base_inf.location --availability-zones role.app.get_availability_zone')]), 'create_autoscale_group': OrderedDict([('create bash', 'aws autoscaling create-auto-scaling-group --auto-scaling-group-name my-lb-asg --launch-configuration-name my-lc --availability-zones role.app.get_availability_zone --load-balancer-names my-load-balancer --max-size 5 --min-size 1 --desired-capacity 2 --region infrastructure.image_base_inf.location')]), 'create_launch_configuration': OrderedDict([('create bash', 'aws autoscaling create-launch-configuration --launch-configuration-name my-lc --image-id resource.create_image_inf.json.ImageId --instance-type infrastructure.image_base_inf.hardware --security-groups role.app.get_security_groups --region infrastructure.image_base_inf.location')]), 'create_image': OrderedDict([('create bash', 'aws ec2 create-image --instance-id infrastructure.image_base_inf.get_id --name DemoApplication3 --description MyDemoApplication --region infrastructure.image_base_inf.location')])})])
        actual = maccli.dao.inheritance.resolve_inheritance(MOCK_BASIC_INHERITANCE, [])
        self.assertEquals(actual, expected)

    def test_single_inheritance(self):
        maccli.logging.basicConfig(level=maccli.logging.DEBUG)
        expected = OrderedDict([('mac', '1.0.0'), ('description', 'Scaled and Load-Balanced Application'), ('name', 'demo'), ('version', '1.0'), ('roles', OrderedDict([('app', OrderedDict([('instance create', OrderedDict([('bootstrap bash', 'sudo apt-get update\nsudo apt-get install nginx -y\n'), ('environment', [OrderedDict([('DB_IP', '127.0.0.1')]), OrderedDict([('APP_BRANCH', 'master')])])]))]))])), ('infrastructures', OrderedDict([('image_base_inf', OrderedDict([('name', 'app'), ('provider', 'amazon'), ('location', 'us-east-1'), ('hardware', 't1.micro'), ('role', 'app'), ('release', 'ubuntu'), ('amount', 1)])), ('create_image_inf', {'resource': 'create_image'}), ('build_lb_inf', OrderedDict([('resource', 'build_lb')])), ('create_launch_configuration_inf', OrderedDict([('resource', 'create_launch_configuration')])), ('create_autoscale_group_inf', OrderedDict([('resource', 'create_autoscale_group')]))])), ('actions', {'get_id': OrderedDict([('ssh', 'wget -q -O - http://169.254.169.254/latest/meta-data/instance-id')]), 'get_security_groups': OrderedDict([('ssh', 'wget -q -O - http://169.254.169.254/latest/meta-data/security-groups')]), 'get_availability_zone': OrderedDict([('ssh', 'wget -q -O - http://169.254.169.254/latest/meta-data/placement/availability-zone')])}), ('resources', {'build_lb': OrderedDict([('create bash', 'aws elb create-load-balancer --load-balancer-name my-load-balancer --listeners Protocol=HTTP,LoadBalancerPort=80,InstanceProtocol=HTTP,InstancePort=80 --region infrastructure.image_base_inf.location --availability-zones role.app.get_availability_zone')]), 'create_autoscale_group': OrderedDict([('create bash', 'aws autoscaling create-auto-scaling-group --auto-scaling-group-name my-lb-asg --launch-configuration-name my-lc --availability-zones role.app.get_availability_zone --load-balancer-names my-load-balancer --max-size 5 --min-size 1 --desired-capacity 2 --region infrastructure.image_base_inf.location')]), 'create_launch_configuration': OrderedDict([('create bash', 'aws autoscaling create-launch-configuration --launch-configuration-name my-lc --image-id resource.create_image_inf.json.ImageId --instance-type infrastructure.image_base_inf.hardware --security-groups role.app.get_security_groups --region infrastructure.image_base_inf.location')]), 'create_image': OrderedDict([('create bash', 'aws ec2 create-image --instance-id infrastructure.image_base_inf.get_id --name DemoApplication3 --description MyDemoApplication --region infrastructure.image_base_inf.location')])})])
        actual = maccli.dao.inheritance.resolve_inheritance(MOCK_SINGLE_INHERITANCE, [])
        self.assertEquals(actual, expected)

    def test_some_infrastructure_default_values(self):
        maccli.logging.basicConfig(level=maccli.logging.DEBUG)
        expected = OrderedDict([('mac', '1.0.0'), ('description', 'Scaled and Load-Balanced Application'), ('name', 'demo'), ('version', '1.0'), ('roles', OrderedDict([('app', OrderedDict([('instance create', OrderedDict([('bootstrap bash', 'sudo apt-get update\nsudo apt-get install nginx -y\n'), ('environment', [OrderedDict([('DB_IP', '127.0.0.1')]), OrderedDict([('APP_BRANCH', 'master')])])]))]))])), ('infrastructures', OrderedDict([('image_base_inf', OrderedDict([('location', 'us-east-1'), ('hardware', 't1.micro'), ('role', 'app'), ('release', 'ubuntu'), ('amount', 1), ('name', 'app'), ('provider', 'amazon')])), ('create_image_inf', {'resource': 'create_image'}), ('build_lb_inf', OrderedDict([('resource', 'build_lb')])), ('create_launch_configuration_inf', OrderedDict([('resource', 'create_launch_configuration')])), ('create_autoscale_group_inf', OrderedDict([('resource', 'create_autoscale_group')]))])), ('actions', {'get_id': OrderedDict([('ssh', 'wget -q -O - http://169.254.169.254/latest/meta-data/instance-id')]), 'get_security_groups': OrderedDict([('ssh', 'wget -q -O - http://169.254.169.254/latest/meta-data/security-groups')]), 'get_availability_zone': OrderedDict([('ssh', 'wget -q -O - http://169.254.169.254/latest/meta-data/placement/availability-zone')])}), ('resources', {'build_lb': OrderedDict([('create bash', 'aws elb create-load-balancer --load-balancer-name my-load-balancer --listeners Protocol=HTTP,LoadBalancerPort=80,InstanceProtocol=HTTP,InstancePort=80 --region infrastructure.image_base_inf.location --availability-zones role.app.get_availability_zone')]), 'create_autoscale_group': OrderedDict([('create bash', 'aws autoscaling create-auto-scaling-group --auto-scaling-group-name my-lb-asg --launch-configuration-name my-lc --availability-zones role.app.get_availability_zone --load-balancer-names my-load-balancer --max-size 5 --min-size 1 --desired-capacity 2 --region infrastructure.image_base_inf.location')]), 'create_launch_configuration': OrderedDict([('create bash', 'aws autoscaling create-launch-configuration --launch-configuration-name my-lc --image-id resource.create_image_inf.json.ImageId --instance-type infrastructure.image_base_inf.hardware --security-groups role.app.get_security_groups --region infrastructure.image_base_inf.location')]), 'create_image': OrderedDict([('create bash', 'aws ec2 create-image --instance-id infrastructure.image_base_inf.get_id --name DemoApplication3 --description MyDemoApplication --region infrastructure.image_base_inf.location')])})])
        actual = maccli.dao.inheritance.resolve_inheritance(MOCK_SOME_VALUES_INHERITANCE, [])
        self.assertEquals(actual, expected)

    def test_override_infrastructure_default_values(self):
        maccli.logging.basicConfig(level=maccli.logging.DEBUG)
        expected = OrderedDict([('mac', '1.0.0'), ('description', 'Scaled and Load-Balanced Application'), ('name', 'demo'), ('version', '1.0'), ('roles', OrderedDict([('app', OrderedDict([('instance create', OrderedDict([('bootstrap bash', 'sudo apt-get update\nsudo apt-get install nginx -y\n'), ('environment', [OrderedDict([('DB_IP', '127.0.0.1')]), OrderedDict([('APP_BRANCH', 'master')])])]))]))])), ('infrastructures', OrderedDict([('image_base_inf', OrderedDict([('name', 'override'), ('location', 'us-east-1'), ('hardware', 't1.micro'), ('role', 'app'), ('release', 'ubuntu'), ('amount', 1), ('provider', 'amazon')])), ('create_image_inf', {'resource': 'create_image'}), ('build_lb_inf', OrderedDict([('resource', 'build_lb')])), ('create_launch_configuration_inf', OrderedDict([('resource', 'create_launch_configuration')])), ('create_autoscale_group_inf', OrderedDict([('resource', 'create_autoscale_group')]))])), ('actions', {'get_id': OrderedDict([('ssh', 'wget -q -O - http://169.254.169.254/latest/meta-data/instance-id')]), 'get_security_groups': OrderedDict([('ssh', 'wget -q -O - http://169.254.169.254/latest/meta-data/security-groups')]), 'get_availability_zone': OrderedDict([('ssh', 'wget -q -O - http://169.254.169.254/latest/meta-data/placement/availability-zone')])}), ('resources', {'build_lb': OrderedDict([('create bash', 'aws elb create-load-balancer --load-balancer-name my-load-balancer --listeners Protocol=HTTP,LoadBalancerPort=80,InstanceProtocol=HTTP,InstancePort=80 --region infrastructure.image_base_inf.location --availability-zones role.app.get_availability_zone')]), 'create_autoscale_group': OrderedDict([('create bash', 'aws autoscaling create-auto-scaling-group --auto-scaling-group-name my-lb-asg --launch-configuration-name my-lc --availability-zones role.app.get_availability_zone --load-balancer-names my-load-balancer --max-size 5 --min-size 1 --desired-capacity 2 --region infrastructure.image_base_inf.location')]), 'create_launch_configuration': OrderedDict([('create bash', 'aws autoscaling create-launch-configuration --launch-configuration-name my-lc --image-id resource.create_image_inf.json.ImageId --instance-type infrastructure.image_base_inf.hardware --security-groups role.app.get_security_groups --region infrastructure.image_base_inf.location')]), 'create_image': OrderedDict([('create bash', 'aws ec2 create-image --instance-id infrastructure.image_base_inf.get_id --name DemoApplication3 --description MyDemoApplication --region infrastructure.image_base_inf.location')])})])
        actual = maccli.dao.inheritance.resolve_inheritance(MOCK_OVERRIDE_VALUES_INHERITANCE, [])
        self.assertEquals(actual, expected)

    def test_override_action(self):
        maccli.logging.basicConfig(level=maccli.logging.DEBUG)
        expected = OrderedDict([('mac', '1.0.0'), ('description', 'Scaled and Load-Balanced Application'), ('name', 'demo'), ('version', '1.0'), ('roles', OrderedDict([('app', OrderedDict([('instance create', OrderedDict([('bootstrap bash', 'sudo apt-get update\nsudo apt-get install nginx -y\n'), ('environment', [OrderedDict([('DB_IP', '127.0.0.1')]), OrderedDict([('APP_BRANCH', 'master')])])]))]))])), ('actions', OrderedDict([('get_id', OrderedDict([('ssh', 'overrided action')])), ('get_availability_zone', OrderedDict([('ssh', 'wget -q -O - http://169.254.169.254/latest/meta-data/placement/availability-zone')])), ('get_security_groups', OrderedDict([('ssh', 'wget -q -O - http://169.254.169.254/latest/meta-data/security-groups')]))])), ('infrastructures', OrderedDict([('image_base_inf', OrderedDict([('name', 'override'), ('location', 'us-east-1'), ('hardware', 't1.micro'), ('role', 'app'), ('release', 'ubuntu'), ('amount', 1), ('provider', 'amazon')])), ('create_image_inf', {'resource': 'create_image'}), ('build_lb_inf', OrderedDict([('resource', 'build_lb')])), ('create_launch_configuration_inf', OrderedDict([('resource', 'create_launch_configuration')])), ('create_autoscale_group_inf', OrderedDict([('resource', 'create_autoscale_group')]))])), ('resources', {'build_lb': OrderedDict([('create bash', 'aws elb create-load-balancer --load-balancer-name my-load-balancer --listeners Protocol=HTTP,LoadBalancerPort=80,InstanceProtocol=HTTP,InstancePort=80 --region infrastructure.image_base_inf.location --availability-zones role.app.get_availability_zone')]), 'create_autoscale_group': OrderedDict([('create bash', 'aws autoscaling create-auto-scaling-group --auto-scaling-group-name my-lb-asg --launch-configuration-name my-lc --availability-zones role.app.get_availability_zone --load-balancer-names my-load-balancer --max-size 5 --min-size 1 --desired-capacity 2 --region infrastructure.image_base_inf.location')]), 'create_launch_configuration': OrderedDict([('create bash', 'aws autoscaling create-launch-configuration --launch-configuration-name my-lc --image-id resource.create_image_inf.json.ImageId --instance-type infrastructure.image_base_inf.hardware --security-groups role.app.get_security_groups --region infrastructure.image_base_inf.location')]), 'create_image': OrderedDict([('create bash', 'aws ec2 create-image --instance-id infrastructure.image_base_inf.get_id --name DemoApplication3 --description MyDemoApplication --region infrastructure.image_base_inf.location')])})])
        actual = maccli.dao.inheritance.resolve_inheritance(MOCK_OVERRIDE_ACTION, [])
        self.assertEquals(actual, expected)

    def test_override_resource(self):
        maccli.logging.basicConfig(level=maccli.logging.DEBUG)
        expected = OrderedDict([('mac', '1.0.0'), ('description', 'Scaled and Load-Balanced Application'), ('name', 'demo'), ('version', '1.0'), ('roles', OrderedDict([('app', OrderedDict([('instance create', OrderedDict([('bootstrap bash', 'sudo apt-get update\nsudo apt-get install nginx -y\n'), ('environment', [OrderedDict([('DB_IP', '127.0.0.1')]), OrderedDict([('APP_BRANCH', 'master')])])]))]))])), ('resources', OrderedDict([('build_lb', OrderedDict([('create bash', 'overrided resource')])), ('create_image', OrderedDict([('create bash', 'aws ec2 create-image --instance-id infrastructure.image_base_inf.get_id --name DemoApplication3 --description MyDemoApplication --region infrastructure.image_base_inf.location')])), ('create_launch_configuration', OrderedDict([('create bash', 'aws autoscaling create-launch-configuration --launch-configuration-name my-lc --image-id resource.create_image_inf.json.ImageId --instance-type infrastructure.image_base_inf.hardware --security-groups role.app.get_security_groups --region infrastructure.image_base_inf.location')])), ('create_autoscale_group', OrderedDict([('create bash', 'aws autoscaling create-auto-scaling-group --auto-scaling-group-name my-lb-asg --launch-configuration-name my-lc --availability-zones role.app.get_availability_zone --load-balancer-names my-load-balancer --max-size 5 --min-size 1 --desired-capacity 2 --region infrastructure.image_base_inf.location')]))])), ('infrastructures', OrderedDict([('image_base_inf', OrderedDict([('name', 'override'), ('location', 'us-east-1'), ('hardware', 't1.micro'), ('role', 'app'), ('release', 'ubuntu'), ('amount', 1), ('provider', 'amazon')])), ('create_image_inf', {'resource': 'create_image'}), ('build_lb_inf', OrderedDict([('resource', 'build_lb')])), ('create_launch_configuration_inf', OrderedDict([('resource', 'create_launch_configuration')])), ('create_autoscale_group_inf', OrderedDict([('resource', 'create_autoscale_group')]))])), ('actions', {'get_id': OrderedDict([('ssh', 'wget -q -O - http://169.254.169.254/latest/meta-data/instance-id')]), 'get_security_groups': OrderedDict([('ssh', 'wget -q -O - http://169.254.169.254/latest/meta-data/security-groups')]), 'get_availability_zone': OrderedDict([('ssh', 'wget -q -O - http://169.254.169.254/latest/meta-data/placement/availability-zone')])})])
        actual = maccli.dao.inheritance.resolve_inheritance(MOCK_OVERRIDE_RESOURCE, [])
        self.assertEquals(actual, expected)

    def test_override_params(self):
        maccli.logging.basicConfig(level=maccli.logging.DEBUG)
        expected = OrderedDict([('mac', '1.0.0'), ('description', 'Scaled and Load-Balanced Application'), ('name', 'demo'), ('version', '1.0'), ('roles', OrderedDict([('app', OrderedDict([('instance create', OrderedDict([('bootstrap bash', 'sudo apt-get update\nsudo apt-get install nginx -y\n'), ('environment', [OrderedDict([('DB_IP', '127.0.0.1')]), OrderedDict([('APP_BRANCH', 'master')])])]))]))])), ('infrastructures', OrderedDict([('create_image_inf', OrderedDict([('params', OrderedDict([('param1', 9), ('param2', 2), ('param3', 3)])), ('resource', 'create_image')]))])), ('actions', {}), ('resources', {'create_image': OrderedDict([('create bash', 'aws ec2 create-image --instance-id infrastructure.image_base_inf.get_id --name DemoApplication3 --description MyDemoApplication --region infrastructure.image_base_inf.location')])})])
        actual = maccli.dao.inheritance.resolve_inheritance(MOCK_OVERRIDE_PARAMS, [])
        self.assertEquals(actual, expected)

    # def test_override_params_1(self):
    #     maccli.logging.basicConfig(level=maccli.logging.DEBUG)
    #     expected = OrderedDict([('mac', '1.0.0'), ('description', 'Scaled and Load-Balanced nginx'), ('name', 'demo'), ('version', 3), ('roles', OrderedDict([('app', OrderedDict([('instance create', OrderedDict([('bootstrap bash', 'sudo apt-get update\nsudo apt-get install nginx -y\n')]))]))])), ('infrastructures', OrderedDict([('image_base_inf', OrderedDict([('name', 'app'), ('provider', 'amazon'), ('location', 'us-east-1'), ('hardware', 't1.micro'), ('role', 'app'), ('release', 'ubuntu:trusty'), ('amount', 1)])), ('build_lb_inf', {'params': OrderedDict([('listeners', 'Protocol=HTTP,LoadBalancerPort=80,InstanceProtocol=HTTP,InstancePort=80'), ('region', 'us-east-1')]), 'resource': 'build_lb'}), ('register_lb_inf', {'params': OrderedDict([('load-balancer-name', 'lb-demo-3'), ('instances', 'role.app.get_id'), ('region', 'us-east-1')]), 'resource': 'register_lb'}), ('create_image_inf', {'ready': 'role.app', 'params': OrderedDict([('region', 'us-east-1')]), 'resource': 'create_image'}), ('create_launch_configuration_inf', {'params': OrderedDict([('region', 'us-east-1')]), 'resource': 'create_launch_configuration'}), ('create_autoscale_group_inf', OrderedDict([('params', OrderedDict([('max-size', 5), ('min-size', 1), ('desired-capacity', 1), ('region', 'us-east-1')])), ('resource', 'create_autoscale_group')])), ('policy_up_inf', OrderedDict([('resource', 'scale_policy'), ('params', OrderedDict([('policy-name', 'ac-up-policy-demo-3'), ('auto-scaling-group-name', 'asg-demo-3'), ('scaling-adjustment', '2'), ('region', 'us-east-1')]))])), ('policy_down_inf', OrderedDict([('resource', 'scale_policy'), ('params', OrderedDict([('policy-name', 'ac-down-policy-demo-3'), ('auto-scaling-group-name', 'asg-demo-3'), ('scaling-adjustment', '-1'), ('region', 'us-east-1')]))])), ('cloudwatch_add_capacity_inf', OrderedDict([('resource', 'cloudwatch'), ('params', OrderedDict([('alarm-name', 'add-capacity-policy-up-demo-3'), ('threshold', '10'), ('comparison-operator', 'GreaterThanOrEqualToThreshold'), ('evaluation-periods', '2'), ('alarm-actions', 'resource.policy_up_inf.json.PolicyARN'), ('region', 'us-east-1')]))])), ('cloudwatch_remove_capacity_inf', OrderedDict([('resource', 'cloudwatch'), ('params', OrderedDict([('alarm-name', 'remove-capacity-policy-down-demo-3'), ('threshold', '3'), ('comparison-operator', 'LessThanOrEqualToThreshold'), ('evaluation-periods', '45'), ('alarm-actions', 'resource.policy_down_inf.json.PolicyARN'), ('region', 'us-east-1')]))]))])), ('actions', {'get_id': OrderedDict([('ssh', 'wget -q -O - http://169.254.169.254/latest/meta-data/instance-id')]), 'get_security_groups': OrderedDict([('ssh', 'wget -q -O - http://169.254.169.254/latest/meta-data/security-groups')]), 'get_availability_zone': OrderedDict([('ssh', 'wget -q -O - http://169.254.169.254/latest/meta-data/placement/availability-zone')])}), ('resources', {'register_lb': OrderedDict([('create bash', 'aws elb register-instances-with-load-balancer --load-balancer-name infrastructure.param.load-balancer-name --instances infrastructure.param.instances --region infrastructure.param.region')]), 'build_lb': OrderedDict([('create bash', 'aws elb create-load-balancer --load-balancer-name lb-demo-3 --listeners infrastructure.param.listeners --region infrastructure.param.region --availability-zones role.app.get_availability_zone'), ('destroy bash', 'aws elb delete-load-balancer --load-balancer-name lb-demo-3 --region infrastructure.param.region')]), 'create_autoscale_group': OrderedDict([('create bash', 'aws autoscaling create-auto-scaling-group --auto-scaling-group-name asg-demo-3 --launch-configuration-name lc-demo-3 --availability-zones role.app.get_availability_zone --load-balancer-names lb-demo-3 --max-size infrastructure.param.max-size --min-size infrastructure.param.min-size --desired-capacity infrastructure.param.desired-capacity --region infrastructure.param.region'), ('destroy bash', '#!/bin/sh\naws autoscaling update-auto-scaling-group --auto-scaling-group-name asg-demo-3 --min-size 0 --max-size 0 --desired-capacity 0 --region us-east-1\nINSTANCES=$(aws autoscaling describe-auto-scaling-groups --auto-scaling-group-names asg-demo-3 --query "AutoScalingGroups[0].Instances[*].InstanceId" --output text --region infrastructure.image_base_inf.location )\nif [ ! -z "$INSTANCES" ]\nthen\n      aws ec2 wait instance-terminated --instance-ids $INSTANCES --region infrastructure.param.region\nfi\nsleep 10\naws autoscaling delete-auto-scaling-group --auto-scaling-group-name asg-demo-3 --force-delete --region infrastructure.param.region\n')]), 'create_launch_configuration': OrderedDict([('create bash', 'aws autoscaling create-launch-configuration --launch-configuration-name lc-demo-3 --image-id resource.create_image_inf.json.ImageId --instance-type infrastructure.image_base_inf.hardware --security-groups role.app.get_security_groups --region infrastructure.param.region'), ('destroy bash', 'aws autoscaling delete-launch-configuration --launch-configuration-name lc-demo-3 --region infrastructure.param.region')]), 'scale_policy': OrderedDict([('create bash', 'aws autoscaling put-scaling-policy --policy-name infrastructure.param.policy-name --auto-scaling-group-name asg-demo-3 --scaling-adjustment infrastructure.param.scaling-adjustment --adjustment-type ChangeInCapacity --cooldown 300 --region infrastructure.param.region')]), 'create_image': OrderedDict([('create bash', 'aws ec2 create-image --instance-id infrastructure.image_base_inf.get_id --name img-demo-3 --description img-demo-3 --region infrastructure.param.region'), ('destroy bash', 'aws ec2 deregister-image --image-id resource.create_image_inf.json.ImageId --region infrastructure.param.region')]), 'cloudwatch': OrderedDict([('create bash', 'aws cloudwatch put-metric-alarm --alarm-name infrastructure.param.alarm-name --metric-name CPUUtilization --namespace AWS/EC2 --statistic Average --period 60 --threshold infrastructure.param.threshold --comparison-operator infrastructure.param.comparison-operator --dimensions "Name=AutoScalingGroupName,Value=asg-demo-3" --evaluation-periods infrastructure.param.evaluation-periods --alarm-actions infrastructure.param.alarm-actions --region infrastructure.param.region'), ('destroy bash', 'aws cloudwatch delete-alarms --alarm-name infrastructure.param.alarm-name --region infrastructure.param.region')])})])
    #     actual = maccli.dao.inheritance.resolve_inheritance(MOCK_OVERRIDE_PARAMS_1, ['INF_NAME=demo', 'INF_VERSION=3', 'AWS_REGION=us-east-1'])
    #     self.assertEquals(actual, expected)

    def test_roles_environment_no_dash(self):
        expected = OrderedDict([('mac', '0.9.19'), ('description', 'Wordpress Containers'), ('name', 'application_1'), ('version', 1.0), ('actions', OrderedDict([('get_id', OrderedDict([('ssh', 'wget -q -O - http://169.254.169.254/latest/meta-data/instance-id')])), ('get_availability_zone', OrderedDict([('ssh', 'wget -q -O - http://169.254.169.254/latest/meta-data/placement/availability-zone')]))])), ('roles', OrderedDict([('wordpress_template', OrderedDict([('instance create', OrderedDict([('branch', 'master'), ('configuration', 'app'), ('environment', OrderedDict([('RDS_FQDN', 'rds-demo.cbhgwxgu29ez.us-east-1.rds.amazonaws.com'), ('RDS_USER', 'myuser'), ('RDS_PASS', 'mysecretpass'), ('WORDPRESS_VERSION', '4.3.1-apache')]))]))]))])), ('infrastructures', OrderedDict([('wordpress_server', OrderedDict([('deployment', 'testing'), ('hardware', 't2.micro'), ('location', 'us-east-1'), ('name', 'wordpress'), ('provider', 'amazon'), ('release', 'any'), ('role', 'wordpress_template'), ('lifespan', 600), ('amount', 1)]))]))])
        actual = maccli.dao.inheritance.resolve_inheritance(MOCK_MACFILE_PARAMS_ROLES_ENVIRONMENT_NO_DASH, [])
        self.assertEquals(actual, expected)



import json
import maccli.helper.http
from maccli.config import RELEASE_ANY
from maccli.view.view_generic import show_error


def get_locations(provider):
    status_code, json, raw = maccli.helper.http.send_request("GET",
                                                             "/provider/locations?provider=%s" % (
                                                                 provider))

    return status_code, json


def get_hardwares(provider, location, release):

    if release is None or release == "":
        release = RELEASE_ANY

    status_code, json, raw = maccli.helper.http.send_request("GET",
                                                             "/provider/hardware?provider=%s&location=%s&release=%s" %
                                                             (provider, location, release))

    if status_code == 400:
        if raw == "No credentials available":
            print ("")
            print ("There is no credentials available in your account for the provider %s" % (provider))
            print ("")
            print (
                "Please login in your account in %s and deploy a production server using the supplier %s" % (
                    maccli.domain, provider))
            print ("")
            print ("You just need to make this action once.")
            print ("")

    return status_code, json


def credentials(provider, client, key):

    params = {
        'provider': provider,
        'client': client,
        'key': key
    }

    json_request = json.dumps(params)

    status_code, json_raw, raw = maccli.helper.http.send_request("POST", "/provider/credentials", data=json_request)

    if status_code == 400:
        show_error("Error while building request: " + raw)

    if status_code == 404:
        show_error("Error while building request: " + raw)

    if status_code == 401:
        show_error("Error while building request: " + raw)

    return status_code, json_raw



import json
import re
import maccli
from maccli.helper.exception import BashException, MacResourceException, InstanceDoesNotExistException
import maccli.helper.macfile
import maccli.helper.cmd
import maccli.helper.metadata
import maccli.dao.api_resource

__author__ = 'tk421'

#
#   Process resource type
#


def run_raw_command(infrastructure_key, command_raw, log_type, key, instances, roles, infrastructures, actions, processed_resources):
    """ Executes a command that will create a resource or run an action """

    maccli.logger.debug("%s %s command_raw: %s " % (log_type, key, command_raw))
    command_clean = command_raw

    # output if the resource has been processed
    is_resource_processed = True
    resource_processed = {}
    is_parsed = True
    if maccli.helper.macfile.has_dependencies(command_raw, roles, infrastructures, actions):
        maccli.logger.debug("Running %s %s with dependency" % (log_type, key))
        try:
            infrastructure = infrastructures[infrastructure_key]
            command_clean, is_parsed = maccli.helper.macfile.parse_envs(command_raw, instances, roles, infrastructures, actions, processed_resources, infrastructure)
        except BashException as e:
            raise MacResourceException("Error: %s\nCommand: %s", e[0], e[1])

        except InstanceDoesNotExistException as e:
            raise MacResourceException("Instance %s  does not exist " % e, "Instance %s  does not exist " % e)
    else:
        maccli.logger.debug("Running %s %s with no dependency" % (log_type, key))

    if is_parsed:
        maccli.logger.debug("%s %s command_clean: %s " % (log_type, key, command_raw))
        rc, stdout, stderr = maccli.helper.cmd.run(command_clean)

        if rc == 0:
            resource_processed = {'stderr': stderr, 'stdout': stdout, 'rc': rc, 'cmd': command_clean}
        else:
            raise MacResourceException("Error while executing resource %s " % infrastructure_key, {'stderr': stderr, 'stdout': stdout, 'rc': rc, 'cmd': command_clean})
    else:
        is_resource_processed = False

    return resource_processed, is_resource_processed


def create_resource(root, infrastructure_key, name, create_bash, rc, stderr, stdout, destroy_bash, infrastructure):
    """
        create a resource
    """
    metadata = maccli.helper.metadata.metadata_resource(root, infrastructure_key, name, infrastructure)
    return maccli.dao.api_resource.create(infrastructure_key, create_bash, rc, stderr, stdout, destroy_bash, metadata)


def update_resource(resource, destroy_bash, rc, stderr, stdout):
    """
        Updates a resource and adds the output for destroying it
    """
    resource['destroy'] = {
        'cmd': destroy_bash,
        'rc': rc,
        'stderr': stderr,
        'stdout': stdout
    }
    return maccli.dao.api_resource.update(resource)


def destroy_resource(resource):
    """
        Destroy the resource
    """
    return maccli.dao.api_resource.delete(resource)


def get_resource_value(inf_name, inf_version, resource_name, key):
    """
        Gets the resource
    """
    resource = maccli.dao.api_resource.get(inf_name, inf_version, resource_name)

    stdout_raw = resource['create']['stdout']
    stdout = ""

    if key is None:
        stdout = stdout_raw
    elif key[:4] == "json":
        stdout_json = json.loads(stdout_raw.strip())
        if key == "json":
            stdout = json.dumps(stdout_json, indent=4)
        elif key[:5] == "json.":
            raw_keys = key[5:].split(".")
            value = stdout_json
            for key in raw_keys:
                if key.isdigit():
                    value = value[int(key)]
                else:
                    value = value[key]

            if isinstance(value, str):
                stdout = value
            else:
                stdout = json.dumps(value, indent=4)

    elif key[:4] == "text":
        # accepted values
        # text
        # text.regex(my_regex)
        # text.regex(my_regex)[1] # witht he position that we are going to extract
        if key == "text":
            stdout = stdout_raw
        elif key[:11] == "text.regex(":
            # format text.regex(my_regex)
            regex_extract = re.compile("text.regex\((.*)\)", re.IGNORECASE)

            # match text
            regex_pattern = regex_extract.findall(key)[0]
            maccli.logger.debug("Searching %s at %s " % (regex_pattern, stdout_raw.strip()))
            regex = re.compile(regex_pattern, re.IGNORECASE)
            matches = regex.findall(stdout_raw.strip())

            # get the proper position
            position_extract_regex = re.compile("text.regex\(.*\)\[([0-9]+)\]", re.IGNORECASE)
            positions = position_extract_regex.findall(key)
            if len(matches) and len(positions):
                stdout = matches[int(positions[0])]
            else:
                stdout = matches

    return stdout


import unittest
import sys
from io import StringIO

import mock

from tests.mock_data import *
import maccli.dao.api_provider


DEFAULT_CONFIGURATION = "cookbook_tag"
DEFAULT_RELEASE = "any"
DEFAULT_PROVIDER = "manageacloud"
DEFAULT_LOCATION = "sfo1"


class AuthTestCase(unittest.TestCase):
    def setUp(self):
        self.stdout = sys.stdout
        sys.stdout = self.buf = StringIO()
        # pass

    def tearDown(self):
        sys.stdout = self.stdout
        # pass

    @mock.patch('maccli.helper.http.send_request')
    def test_locations(self, mock):
        mock.return_value = (200, MOCK_LOCATION_LIST_JSON, MOCK_LOCATION_LIST_JSON_RAW)
        status_code, json_response = maccli.dao.api_provider.get_locations(DEFAULT_PROVIDER)
        mock.assert_called_once_with("GET", "/provider/locations?provider=%s" % (
        DEFAULT_PROVIDER))
        self.assertEqual(json_response, MOCK_LOCATION_LIST_JSON)
        self.assertEqual(status_code, 200)

    @mock.patch('maccli.helper.http.send_request')
    def test_hardwares_ok(self, mock):
        mock.return_value = (200, MOCK_HARDWARE_LIST_JSON, MOCK_HARDWARE_LIST_JSON_RAW)
        status_code, json_response = maccli.dao.api_provider.get_hardwares(DEFAULT_PROVIDER, DEFAULT_LOCATION, DEFAULT_RELEASE)
        mock.assert_called_once_with("GET", "/provider/hardware?provider=%s&location=%s&release=%s" % (
        DEFAULT_PROVIDER, DEFAULT_LOCATION, DEFAULT_RELEASE))
        self.assertEqual(json_response, MOCK_HARDWARE_LIST_JSON)
        self.assertEqual(status_code, 200)

    @mock.patch('maccli.helper.http.send_request')
    def test_hardwares_error(self, mock):
        mock.return_value = (400, None, "No credentials available")
        status_code, json_response = maccli.dao.api_provider.get_hardwares(DEFAULT_PROVIDER, DEFAULT_LOCATION, DEFAULT_RELEASE)
        mock.assert_called_once_with("GET", "/provider/hardware?provider=%s&location=%s&release=%s" % (
        DEFAULT_PROVIDER, DEFAULT_LOCATION, DEFAULT_RELEASE))
        self.assertEqual(json_response, None)
        self.assertEqual(status_code, 400)
        out = self.buf.getvalue()
        self.assertEqual(' '.join(OUTPUT_HARDWARES_NO_CREDENTIALS.split()), ' '.join(out.split()))



import unittest

import mock

from tests.mock_data import *
import maccli.command_cli
import maccli.dao.api_instance


class AuthTestCase(unittest.TestCase):
    @mock.patch('maccli.helper.http.send_request')
    def test_dao_instance_list_command(self, mock_get_instance_list):
        mock_get_instance_list.return_value = 200, MOCK_INSTANCE_LIST_JSON, MOCK_INSTANCE_LIST_JSON_RAW
        json = maccli.dao.api_instance.get_list()
        self.assertEqual(MOCK_INSTANCE_LIST_JSON, json)


    @mock.patch('maccli.service.instance.list_instances')
    def test_cli_command_instance_list(self, mock_list_instances):
        mock_list_instances.return_value = MOCK_INSTANCE_LIST_JSON
        maccli.command_cli.instance_list()




from prettytable import PrettyTable


def show_locations(locations):
    pretty = PrettyTable(["Distribution", "Location", "Description"])
    pretty.align["Description"] = "l"
    for location in locations:
        pretty.add_row([location['release'], location['id'], location['description']])

    print(pretty)



class MacApiError(Exception):
    """An error status code was returned when querying the HTTP API"""
    pass


class MacAuthError(MacApiError):
    """An 401 Unauthorized status code was returned when querying the API"""
    pass


class MacParamValidationError(MacApiError):
    """There is an error when validating params for infrastructures"""
    pass


class MacParseEnvException(Exception):
    """Error while parsing variables in a macfile"""
    pass


class MacParseParamException(Exception):
    """Error while parsing params in a macfile"""
    pass


class MacResourceException(Exception):
    """Error while creating a resource"""
    pass


class InstanceDoesNotExistException(Exception):
    """The selected instance does not exist"""
    pass


class InstanceNotReadyException(Exception):
    """Instance is not ready"""
    pass


class MacErrorCreatingTier(Exception):
    """Error while parsing variables in a macfile"""
    pass


class FactError(Exception):
    """ Error in the Fact  """
    pass


class InternalError(RuntimeError):
    pass


class BashException(Exception):
    """Bash command failed running"""
    pass


class MacParameterNotFound(Exception):
    """Parameter has not been found"""
    pass


class MacJsonException(Exception):
    """Parameter has not been found"""
    pass



#
#   Interactive session is a preview feature.
#   Please use with care.
#

import shlex
from prompt_toolkit.styles import DefaultStyle
from maccli.helper.exception import InternalError
from maccli.view.view_generic import show

__author__ = 'tk421'

from prompt_toolkit import prompt, AbortAction
from prompt_toolkit.history import InMemoryHistory
from prompt_toolkit.contrib.completers import WordCompleter
from prompt_toolkit.auto_suggest import AutoSuggestFromHistory
from prompt_toolkit.contrib.regular_languages.completion import GrammarCompleter
from prompt_toolkit.contrib.regular_languages.lexer import GrammarLexer
from prompt_toolkit.layout.lexers import SimpleLexer
from prompt_toolkit.contrib.regular_languages.compiler import compile
import maccli.mac_cli
import maccli

from pygments.token import Token


op_mac = ['mac']
op_main = ['login', 'instance', 'configuration', 'infrastructure']
op_instance = ['list', 'create', 'update', 'destroy', 'ssh', 'facts', 'log', 'lifespan']
op_configuration = ['list', 'search']
op_infrastructure = ['macfile', 'list', 'items', 'destroy', 'update', 'lifespan', 'sshkey']


def create_grammar():
    return compile("""
        ((?P<op_mac>mac)    \s*  (?P<op_main>login)   \s+) |
        ((?P<op_mac>mac)    \s*  (?P<op_main>instance)   \s+   (?P<op_instance>[list|create|update|destroy|ssh|facts|log|lifespan]+)    \s+    (?P<op_parameter>("[^"]*"|[^"]+)(\s+|$))    \s+) |
        ((?P<op_mac>mac)    \s*  (?P<op_main>configuration)   \s+   (?P<op_configuration>[list|search]+)    \s+    (?P<op_parameter>("[^"]*"|[^"]+)(\s+|$))    \s+) |
        ((?P<op_mac>mac)    \s*  (?P<op_main>infrastructure)   \s+   (?P<op_infrastructure>[macfile|list|items|destroy|update|lifespan|sshkey]+)    \s+    (?P<op_parameter>("[^"]*"|[^"]+)(\s+|$))    \s+)
    """)


class MacStyle(DefaultStyle):
    styles = {}
    styles.update(DefaultStyle.styles)
    styles.update({
        Token.Operator: '#33aa33 bold',
        Token.Number: '#aa3333 bold',
        Token.Text: '#7A8DF5 bold',

        Token.TrailingInput: 'bg:#662222 #ffffff',
        })


def start():

    g = create_grammar()

    lexer = GrammarLexer(g, lexers={
        'op_mac': SimpleLexer(Token.Operator),
        'op_main': SimpleLexer(Token.Operator),
        'op_instance': SimpleLexer(Token.Operator),
        'op_configuration': SimpleLexer(Token.Operator),
        'op_infrastructure': SimpleLexer(Token.Operator),
        'op_parameter': SimpleLexer(Token.Text),
        })

    completer = GrammarCompleter(g, {
        'op_main': WordCompleter(op_main),
        'op_instance': WordCompleter(op_instance),
        'op_configuration': WordCompleter(op_configuration),
        'op_infrastructure': WordCompleter(op_infrastructure),
        })

    history = InMemoryHistory()

    parser = maccli.mac_cli.initialize_parser()

    show("Start typing 'mac', CTRL+C to exit")
    user_aborted = False
    program_running = True
    while program_running:
        try:
            text = prompt('> ', lexer=lexer, completer=completer, style=MacStyle, history=history, auto_suggest=AutoSuggestFromHistory())
            argv_raw = shlex.split(text)
            argv = maccli.mac_cli.patch_help_option(argv_raw)
            args = parser.parse_args(argv)
            maccli.mac_cli.dispatch_cmds(args)
            user_aborted = False
        except InternalError as e:
            maccli.logger.debug("Code raised Internal Error", e)
            pass
        except EOFError as e:
            maccli.logger.debug("Code raised EOFError", e)
            pass
        except KeyboardInterrupt as e:
            maccli.logger.debug("Code raised KeyboardInterrupt", e)
            if user_aborted:
                program_running = False
            else:
                user_aborted = True
                show("Press CTRL+C again to exit")
            pass







import unittest
import sys
from io import StringIO

import mock

import maccli.dao.api_instance
from tests.mock_data import *


DEFAULT_DEPLOYMENT = "testing"
DEFAULT_PROVIDER = "manageacloud"
DEFAULT_BRANCH = "master"
DEFAULT_RELEASE = "any"
DEFAULT_RELEASE_VERSION = ""
DEFAULT_CONFIGURATION = "cookbook_tag"
DEFAULT_BOOTSTRAP = ""
DEFAULT_LOCATION = "sfo1"
DEFAULT_SERVERNAME = "server_name"
DEFAULT_SERVER_ID = "serverid"
DEFAULT_HARDWARE = "512mb"
DEFAULT_LIFESPAN = 90
DEFAULT_ENVIRONMENT = ["KEY=VALUE"]
DEFAULT_HD = ["/dev/sda1:100"]
DEFAULT_PORT = [22]
DEFAULT_NET = ""
DEFAULT_METADATA = {'infrastructure': {
                       'version': '1.0infrastructure_version',
                       'name': 'infrastructure name',
                       'macfile_role_name': 'database',
                       'macfile_infrastructure_name': 'database_master',
                     }
                   }
DEFAULT_APPLYCHANGES = True

class AuthTestCase(unittest.TestCase):
    def setUp(self):
        self.stderr = sys.stderr
        sys.stderr = self.buf = StringIO()
        # pass

    def tearDown(self):
        sys.stderr = self.stderr
        # pass

    @mock.patch('maccli.helper.http.send_request')
    def test_instance_create(self, mock):
        mock.return_value = (200, MOCK_RESPONSE_INSTANCE_CREATE_JSON, MOCK_RESPONSE_INSTANCE_CREATE_JSON_RAW)
        json_response = maccli.dao.api_instance.create(DEFAULT_CONFIGURATION, DEFAULT_BOOTSTRAP, DEFAULT_DEPLOYMENT, DEFAULT_LOCATION,
                                                       DEFAULT_SERVERNAME,
                                                       DEFAULT_PROVIDER, DEFAULT_RELEASE, DEFAULT_RELEASE_VERSION, DEFAULT_BRANCH,
                                                       DEFAULT_HARDWARE, DEFAULT_LIFESPAN, DEFAULT_ENVIRONMENT,
                                                       DEFAULT_HD, DEFAULT_PORT, DEFAULT_NET, DEFAULT_METADATA, DEFAULT_APPLYCHANGES)
        # TODO: python2 to python3 assertion fails due order or parameters
        #mock.assert_called_once_with("POST", "/instance", data=MOCK_INSTANCE_CREATE_PARAMETERS_JSON_RAW)
        error = self.buf.getvalue()
        self.assertEqual(' '.join("".split()), ' '.join(error.split()))
        self.assertEqual(json_response, MOCK_RESPONSE_INSTANCE_CREATE_JSON)


    @mock.patch('maccli.helper.http.send_request')
    def test_instance_create_error(self, mock):
        mock.return_value = (400, None, "Error Response")
        json_response = maccli.dao.api_instance.create(DEFAULT_CONFIGURATION, DEFAULT_BOOTSTRAP, DEFAULT_DEPLOYMENT, DEFAULT_LOCATION,
                                                       DEFAULT_SERVERNAME,
                                                       DEFAULT_PROVIDER, DEFAULT_RELEASE, DEFAULT_RELEASE_VERSION, DEFAULT_BRANCH,
                                                       DEFAULT_HARDWARE, DEFAULT_LIFESPAN, DEFAULT_ENVIRONMENT,
                                                       DEFAULT_HD, DEFAULT_PORT, DEFAULT_NET, DEFAULT_METADATA, DEFAULT_APPLYCHANGES)
        #TODO python 2 to python3: assertion fails due order of parameters
        #mock.assert_called_once_with("POST", "/instance", data=MOCK_INSTANCE_CREATE_PARAMETERS_JSON_RAW)
        error = self.buf.getvalue()
        self.assertEqual(' '.join(MOCK_RESPONSE_INSTANCE_CREATE_ERROR.split()), ' '.join(error.split()))
        self.assertEqual(json_response, None)

    @mock.patch('maccli.helper.http.send_request')
    def test_instance_destroy_error(self, mock):
        mock.return_value = (400, None, "Error Response")
        json_response = maccli.dao.api_instance.destroy(DEFAULT_SERVER_ID)
        mock.assert_called_once_with("DELETE", "/instance/%s" % DEFAULT_SERVER_ID)
        error = self.buf.getvalue()
        self.assertEqual(' '.join(MOCK_RESPONSE_INSTANCE_DESTROY_ERROR.split()), ' '.join(error.split()))
        self.assertEqual(json_response, None)

    @mock.patch('maccli.helper.http.send_request')
    def test_instance_destroy_error_not_found(self, mock):
        mock.return_value = (404, None, "Error Response")
        json_response = maccli.dao.api_instance.destroy(DEFAULT_SERVERNAME)
        mock.assert_called_once_with("DELETE", "/instance/%s" % DEFAULT_SERVERNAME)
        error = self.buf.getvalue()
        self.assertEqual(' '.join(MOCK_RESPONSE_INSTANCE_DESTROY_NOT_FOUND.split()), ' '.join(error.split()))
        self.assertEqual(json_response, None)

    @mock.patch('maccli.helper.http.send_request')
    def test_instance_destroy_ok(self, mock):
        mock.return_value = (200, MOCK_RESPONSE_INSTANCE_DESTROY_JSON, MOCK_RESPONSE_INSTANCE_DESTROY_JSON_RAW)
        json_response = maccli.dao.api_instance.destroy(DEFAULT_SERVERNAME)
        mock.assert_called_once_with("DELETE", "/instance/%s" % DEFAULT_SERVERNAME)
        self.assertEqual(json_response, MOCK_RESPONSE_INSTANCE_DESTROY_JSON)

    @mock.patch('maccli.helper.http.send_request')
    def test_instance_list(self, mock):
        mock.return_value = (200, MOCK_RESPONSE_INSTANCE_LIST_JSON, MOCK_RESPONSE_INSTANCE_LIST_JSON_RAW)
        json_response = maccli.dao.api_instance.get_list()
        mock.assert_called_once_with("GET", "/instances")
        self.assertEqual(json_response, MOCK_RESPONSE_INSTANCE_LIST_JSON)



import os
import configparser

import base64
import maccli.helper.http


def get_auth(username, password):
    """Returns the user's Username and ApiKey, or raises an exception if username/password incorrect

    :param username: The email of the user to authenticate
    :type username: str
    :param password: The password of the user to authenticate
    :type password: str
    :raises: MacApiError
    :returns: str, str -- the Username, ApiKey to use for the given username/email
    """
    credentials_bytes = ('%s:%s' % (username, password)).encode('ascii')
    base64string = base64.b64encode(credentials_bytes).decode('utf-8', 'ignore').replace('\n', '')
    status_code, json, raw = maccli.helper.http.send_request("GET", "/user/auth", auth=base64string)
    user = username
    apikey = None
    if json:
        user = json.get('username', None)
        apikey = json.get('apiKey', None)
    return user, apikey


def load_from_file(file="~/.manageacloud"):
    """Attempts to read manageacloud credentials from a config file and return a tuple of (user, apikey)

    :param file: The filename where Manageacloud auth config is stored
    :type file: str
    :returns: tuple -- tuple of (user, apikey) if config found and valid, (None, None) otherwise
    """
    try:
        cfgfile = os.path.expanduser(file)
        cp = configparser.ConfigParser()
        cp.read(cfgfile)
        return cp.get("auth", "user"), cp.get("auth", "apikey")
    except configparser.Error:
        return None, None


import unittest
from maccli.helper.unsortable import ordered_load, yaml

from tests.mock_data import *


class UnsortableTestCase(unittest.TestCase):

    def test_int_as_string(self):
        mock_yaml = "test: 2"
        expected = OrderedDict([(u'test', u'2')])
        raw = ordered_load(mock_yaml, yaml.BaseLoader)
        self.assertEqual(raw, expected)




from collections import OrderedDict
from maccli.helper.exception import MacParseParamException
from maccli.helper.unsortable import ordered_load, yaml
import maccli.helper.macfile


def resolve_inheritance(macfile_str, params):
    """
    Loads the macfile and solves the inheritance
    """
    params_found = []
    raw, found = maccli.helper.macfile.parse_params(macfile_str, params)
    params_found += found
    raw = ordered_load(raw, yaml.SafeLoader)

    if 'parents' in raw:  # there is inheritance

        # load the parents. For now, only one level of inheritance is allowed
        parents_raw = {}
        for parent in raw['parents']:
            parent_raw = maccli.helper.macfile.load_macfile(raw['parents'][parent])
            parent_raw, found = maccli.helper.macfile.parse_params(parent_raw, params)
            params_found += found
            parents_raw[parent] = ordered_load(parent_raw, yaml.SafeLoader)

        # lets read the original file and paste all the information from the parent class
        if 'actions' not in raw:
            raw['actions'] = {}  # order is not important here

        if 'resources' not in raw:
            raw['resources'] = {}  # order is not important here

        # add 'resources' and 'actions' from parents
        for key in parents_raw:
            parent_raw = parents_raw[key]

            if 'actions' in parent_raw:
                for action in parent_raw['actions']:
                    if action in raw['actions']:
                        maccli.logger.info("Action %s exists, it wont be overwritten by %s.%s", action, key, action)
                    else:
                        raw['actions'][action] = parent_raw['actions'][action]

            if 'resources' in parent_raw:
                for resource in parent_raw['resources']:
                    if resource in raw['resources']:
                        maccli.logger.info("Resource %s exists, it wont be overwritten by %s.%s", resource, key, resource)
                    else:
                        raw['resources'][resource] = parent_raw['resources'][resource]


        """
            To fix the infrastructure section, we have to reconstruct it by using the "defaults" from the parent
            Format:
              # Overwriting a section
              # single case
              aws.image_base_inf:
                name: app
                provider: amazon
                location: us-east-1
                hardware: t1.micro
                role: app
                release: ubuntu
                amount: 1

               # Appending a single section using the defaults
               # single case
               aws.image_base_inf:

               # Appending all the sections that has not been defined already, using the defaults
               # wildcard case
               aws.*
        """

        if 'infrastructures' in raw:

            clean_infrastructures = OrderedDict({})
            for key in raw['infrastructures']:
                raw_values = raw['infrastructures'][key]
                if raw_values is None:
                    raw_values = {}

                if key[-2:] == '.*':
                    # wildcard case
                    parent, section = key.split(".")
                    default_infrastructures = parents_raw[parent]['defaults']
                    for default_infrastructure_key in default_infrastructures:
                        default_infrastructure_values = default_infrastructures[default_infrastructure_key]
                        if not (default_infrastructure_key in raw['infrastructures'] or parent + '.' + default_infrastructure_key in raw['infrastructures']):
                            maccli.logger.debug("Inheritance wildcard adds to infrastructure %s", default_infrastructure_key)
                            clean_infrastructures.update({default_infrastructure_key: default_infrastructure_values})

                elif '.' in key:
                    # single case
                    parent, section = key.split(".")
                    default_values = parents_raw[parent]['defaults'][section]

                    if default_values is not None:
                        for default_key in default_values:
                            default_value = default_values[default_key]
                            if default_key not in raw_values:  # add the key
                                raw_values[default_key] = default_value
                            else:  # merge the key
                                raw_values[default_key] = _merge_dictionaries(default_value, raw_values[default_key])

                    maccli.logger.debug("Inheritance single adds to infrastructure %s", section)
                    clean_infrastructures.update({section: raw_values})
                else:
                    # no inheritance work
                    maccli.logger.debug("Inheritance adds to infrastructure %s", key)
                    clean_infrastructures.update({key: raw_values})

            maccli.logger.debug("Inheritance defines clean infrastructure %s", clean_infrastructures)
            raw['infrastructures'] = clean_infrastructures

        if len(set(params_found)) != len(set(params)):
            # remove found parameters
            missing_parameters = [x for x in params if x.split("=")[0] not in params_found]
            raise MacParseParamException("Variable(s) %s could not be found in macfile" % missing_parameters)

        # remove the parents
        del raw['parents']

    return raw


def _merge_dictionaries(base, new_values):
    if isinstance(new_values, str):
        total = new_values
    else:
        total = base
        for key in new_values:
            value = new_values[key]
            total[key] = value

    return total


import urllib

import maccli.helper.http


def get_infrastructure_list():
    status_code, json, raw = maccli.helper.http.send_request("GET", "/infrastructure")

    return status_code, json


def search_infrastructure(name, version):

    params = {}
    if name is not None:
        params['name'] = name

    if version is not None:
        params['version'] = version

    querystring = "?" + urllib.parse.urlencode(params, True)

    status_code, json_response, raw = maccli.helper.http.send_request("GET", "/infrastructure%s" % querystring)

    return status_code, json_response




import sys
import logging
import codecs
import copy
import argparse

import maccli
import maccli.parser_cli
import maccli.command_cli
#import maccli.lib.
#from maccli.lib.aliases import AliasedSubParsersAction
#import maccli.interactive

from maccli.helper.exception import InternalError
from maccli.view.view_generic import show_error


# TODO lib aliases is kind of failing ... not sure why
class AliasedSubParsersAction(argparse._SubParsersAction):
    old_init = staticmethod(argparse._ActionsContainer.__init__)

    @staticmethod
    def _containerInit(self, description, prefix_chars, argument_default, conflict_handler):
        AliasedSubParsersAction.old_init(self, description, prefix_chars, argument_default, conflict_handler)
        self.register('action', 'parsers', AliasedSubParsersAction)

    class _AliasedPseudoAction(argparse.Action):
        def __init__(self, name, aliases, help):
            dest = name
            if aliases:
                dest += ' (%s)' % ','.join(aliases)
            sup = super(AliasedSubParsersAction._AliasedPseudoAction, self)
            sup.__init__(option_strings=[], dest=dest, help=help)

    def add_parser(self, name, **kwargs):
        aliases = kwargs.pop('aliases', [])
        parser = super(AliasedSubParsersAction, self).add_parser(name, **kwargs)

        # Make the aliases work.
        for alias in aliases:
            self._name_parser_map[alias] = parser
        # Make the help text reflect them, first removing old help entry.
        if 'help' in kwargs:
            help = kwargs.pop('help')
            self._choices_actions.pop()
            pseudo_action = self._AliasedPseudoAction(name, aliases, help)
            self._choices_actions.append(pseudo_action)

        return parser



def initialize_parser():
    # Top parser
    parser = argparse.ArgumentParser(description="Manageacloud.com CLI", prog='mac')
    parser.register('action', 'parsers', AliasedSubParsersAction)
    parser.add_argument('--version', action='version', version='%(prog)s ' + maccli.__version__)
    parser.add_argument('-v', '--verbose', action='store_true', help="Show verbose information")
    parser.add_argument('-q', '--quiet', action='store_true', help="Enable loggable output")
    parser.add_argument('-s', '--disable-strict-host', action='store_true', help="Disable Strict Host Key Checking. It sets the SSH parameter -o 'StrictHostKeyChecking no' when connecting to the instances.")
    parser.add_argument('--debug', action='store_true', help="Enable debug")
    #parser.add_argument('-i', '--interactive', action='store_true', dest="interactive", help="Interactive session - preview feature")
    subparsers = parser.add_subparsers(title="mac's CLI commands", dest='cmd')
    maccli.parser_cli.add_login_parser(subparsers)
    maccli.parser_cli.add_instance_parser(subparsers)
    maccli.parser_cli.add_resource_parser(subparsers)
    maccli.parser_cli.add_configuration_parser(subparsers)
    maccli.parser_cli.add_infrastructure_parser(subparsers)
    maccli.parser_cli.add_provider_parser(subparsers)
    return parser


def patch_help_option(argv=sys.argv):
    args = copy.copy(argv)

    if not args:
        raise InternalError("Wrong argument is set, cannot be empty")

    if len(args) == 1:
        maccli.command_cli.help()

    if len(args) == 2:
        if args[1] == 'instance':
            maccli.command_cli.instance_help()

        if args[1] == 'configuration':
            maccli.command_cli.configuration_help()

        if args[1] == 'provider':
            maccli.command_cli.provider_help()

    return args[1:]


def dispatch_cmds(args):

    if args.debug:
        logging.basicConfig(level=logging.DEBUG)
    elif args.verbose:
        logging.basicConfig(level=logging.INFO)
    else:
        logging.basicConfig(level=logging.WARN)

    maccli.quiet = args.quiet
    maccli.disable_strict_host_check = args.disable_strict_host

    maccli.logger.debug("Args options %s: " % args)

    if args.cmd == 'login':
        maccli.command_cli.login()

    elif maccli.user is None:
        maccli.command_cli.no_credentials()

    elif args.cmd == 'instance' or args.cmd == 'ins':

        if args.subcmd == 'create':
            if args.yaml:
                maccli.command_cli.convert_to_yaml(args)
            else:
                maccli.command_cli.instance_create(args.configuration, args.bootstrap, args.deployment, args.location, args.name,
                                                   #args.provider, args.release, args.release_version, args.branch, args.hardware, args.lifespan,
                                                   args.provider, args.release, args.release_version, "master", args.hardware, args.lifespan,
                                                   args.environment, args.hd, args.net, args.port)
        elif args.subcmd == 'update':
            if args.id is None:
                show_error("Parameter 'id' is required.")
                maccli.command_cli.instance_update_help()
            else:
                maccli.command_cli.instance_update(args.id, args.configuration, args.bootstrap)

        elif args.subcmd == 'destroy':
            if args.id is None:
                show_error("Parameter 'id' is required.")
                maccli.command_cli.instance_destroy_help()
            else:
                maccli.command_cli.instance_destroy(args.id)

        elif args.subcmd == 'list':
            maccli.command_cli.instance_list()

        elif args.subcmd == 'facts':
            maccli.command_cli.instance_fact(args.id)

        elif args.subcmd == 'log':
            maccli.command_cli.instance_log(args.id, args.follow)

        elif args.subcmd == 'lifespan':
            maccli.command_cli.instance_lifespan(args.id, args.amount)

        elif args.subcmd == 'ssh':
            if args.id is None:
                show_error("Parameter 'id' is required.")
                maccli.command_cli.instance_ssh_help()
            else:
                if args.command is not None:
                    command = " ".join(args.command)
                else:
                    command = args.command

                maccli.command_cli.instance_ssh(args.id, command)

    elif args.cmd == "configuration":
        if args.subcmd == 'list':
            maccli.command_cli.configuration_list()
        elif args.subcmd == 'search':
            maccli.command_cli.configuration_search(args.keyword, args.url)

    elif args.cmd == "infrastructure" or args.cmd == 'infra':
        if args.subcmd == 'list':
            maccli.command_cli.infrastructure_list()
        elif args.subcmd == 'macfile':
            maccli.command_cli.process_macfile(args.file[0], args.resume, args.param, args.quiet, args.on_failure)
        elif args.subcmd == 'items':
            maccli.command_cli.infrastructure_search(args.name, args.version)
        elif args.subcmd == 'destroy':
            maccli.command_cli.infrastructure_destroy(args.name, args.version)
        elif args.subcmd == 'update':
            maccli.command_cli.infrastructure_update(args.name, args.version, args.configuration)
        elif args.subcmd == 'lifespan':
            maccli.command_cli.infrastructure_lifespan(args.amount, args.name, args.version)
        elif args.subcmd == 'sshkey':
            maccli.command_cli.infrastructure_ssh_keys(args.name, args.version, args.known_host)

    elif args.cmd == "provider":
        if args.subcmd == 'credential':
            maccli.command_cli.credentials(args.provider, args.clientid, args.key, args.force_file)

    elif args.cmd == "resource":
        if args.subcmd == 'get_stdout':
            maccli.command_cli.resouce_get_stdout(args.infrastructure_name, args.infrastructure_version, args.resource_name, args.key)


def parse_args(self, args):
    # factored out for testability
    return self.parser.parse_args(args)


def main():
    parser = initialize_parser()
    argv = patch_help_option(sys.argv)
    #if argv == ['-i']:  # interactive session
    #    maccli.interactive.start()
    #else:  # Normal session
    args = parser.parse_args(argv)
    dispatch_cmds(args)


if __name__ == "__main__":
    main()


import time
import maccli

import pprint
from maccli.config import MACFILE_ON_FAILURE_DESTROY_OTHERS, MACFILE_ON_FAILURE_DESTROY_ALL
from maccli.view.view_generic import GREEN, RED
import maccli.service.instance
import maccli.view.view_generic
import maccli.view.view_instance
import maccli.view.view_infrastructure
import maccli.helper.macfile
import maccli.helper.cmd
import maccli.service.resource
from maccli.helper.network import is_ip_private, is_local
from maccli.helper.exception import MacParseEnvException, FactError, MacApiError, BashException, \
    MacResourceException

pp = pprint.PrettyPrinter(indent=4)


def _get_private_ip_from_fatcs(facts):
    if facts is not None and 'interfaces' in facts:
        interfaces = facts['interfaces'].split(",")
        priv_ip = ""
        fallback_ip = ""
        for interface_key in interfaces:
            ip_raw = facts["ipaddress_%s" % interface_key]
            if is_ip_private(ip_raw) and not is_local(ip_raw):
                priv_ip = ip_raw
                break
            elif fallback_ip == "" and not is_local(ip_raw):
                fallback_ip = ip_raw

        if priv_ip != "":
            ip = priv_ip
        else:
            ip = fallback_ip
    else:
        raise FactError("Interfaces not available")

    return ip


def _get_environment(role, infrastructure):
    environment = None
    try:
        if 'environment' in infrastructure.keys():
            environment = infrastructure["environment"]

        if 'environment' in role.keys():
            if environment is not None:
                environment = environment + role["environment"]
            else:
                environment = role["environment"]

    except KeyError:
        pass

    return environment


def parse_instance_envs(env_raws, instances):
    """
        Replaces the values in the environment variables for values
        that exist in this infrastructure

        Supported:

         - role.PUBLIC_IP -> adds the public ip
         - role.PRIVATE_IP -> adds the first private ip retrieved by facts
         - role.FACT.FACTNAME -> replace fact by any other fact

    :param role:
    :param roles_created:
    :return:
    """

    envs_clean = {}
    all_processed = True
    for env in env_raws:
        key = list(env.keys())[0]
        val = env[key]
        if isinstance(val, (int)):
            val = str(val)

        if isinstance(val, str):
            if val.endswith(".PUBLIC_IP"):
                """
                    We substitute the value by the given role
                    Format existing_role.PROPERTY
                """
                maccli.logger.info(".PUBLIC_IP fact (%s)" % val)
                role_name, property = val.split(".", 1)
                try:
                    ips = []
                    for instance in instances:
                        instance_role_name = instance['metadata']['infrastructure']['macfile_role_name']
                        instance_infrastructure_name = instance['metadata']['infrastructure']['macfile_infrastructure_name']

                        ip = instance['ipv4']
                        if ip and (instance_role_name == role_name or instance_infrastructure_name == role_name):
                            ips.append(ip)

                    if len(ips) > 0:
                        envs_clean[key] = ips
                    else:
                        all_processed = False
                except KeyError:
                    raise MacParseEnvException("Error while parsing env PUBLIC_IP", key, val)

            elif val.endswith(".PRIVATE_IP"):
                maccli.logger.info(".PRIVATE_IP fact (%s)" % val)
                role_name, property = val.split(".", 1)
                try:
                    ips = []
                    is_error = False
                    for instance in instances:
                        instance_role_name = instance['metadata']['infrastructure']['macfile_role_name']
                        instance_infrastructure_name = instance['metadata']['infrastructure']['macfile_infrastructure_name']
                        if instance_role_name == role_name or instance_infrastructure_name == role_name:
                            try:
                                instance_facts = maccli.service.instance.facts(instance['id'])
                                ip = _get_private_ip_from_fatcs(instance_facts)
                                ips.append(ip)
                            except FactError:
                                is_error = True
                                pass
                            except MacApiError:
                                is_error = True
                                pass

                    if len(ips) > 0 and not is_error:
                        envs_clean[key] = ips
                    else:
                        all_processed = False

                except KeyError:
                    raise MacParseEnvException("Error while parsing env PRIVATE_IP", key, val)

            elif ".FACT." in val:
                maccli.logger.info(".FACT. fact (%s)" % val)
                role_name, fact, property = val.split(".", 2)

                try:
                    facts = []
                    is_error = False
                    for instance in instances:
                        instance_role_name = instance['metadata']['infrastructure']['macfile_role_name']
                        instance_infrastructure_name = instance['metadata']['infrastructure']['macfile_infrastructure_name']

                        if instance_role_name == role_name or instance_infrastructure_name == role_name:
                            try:
                                instance_facts = maccli.service.instance.facts(instance['id'])
                                property_value = instance_facts[property.lower()]
                                facts.append(property_value)
                            except FactError:
                                is_error = True
                                pass
                            except MacApiError:
                                is_error = True
                                pass

                    if len(facts) > 0 and not is_error:
                        envs_clean[key] = facts
                    else:
                        all_processed = False

                except KeyError:
                    raise MacParseEnvException("Error while parsing env PRIVATE_IP", key, val)

            else:
                envs_clean[key] = val

    return envs_clean, all_processed


def apply_instance_infrastructure_changes(instances, infrastructure_name, version, quiet, infrastructures, infrastructure_resources_processed):
    """
    Make sure that all instances in a infrastructure are processed
    :param instances:
    :return:
    """
    configuration_pending = []
    other_instances = []
    for instance in instances:

        if instance['status'] == "Instance completed":
            infrastructure_key = instance['metadata']['infrastructure']['macfile_infrastructure_name']
            infrastructure = infrastructures[infrastructure_key]
            dependencies_ready = maccli.helper.macfile.is_role_dependencies_ready(infrastructure, instances, infrastructure_key)
            if dependencies_ready:
                configuration_pending.append(instance)
            else:
                maccli.logger.debug("[%s] instance discarded as 'ready' dependencies are not met" % instance['servername'])

        if instance['ipv4'] is not None:
            other_instances.append(instance)

    action = False
    for instance in configuration_pending:
        maccli.logger.debug("[%s] Checking instance " % instance['servername'])

        #
        #    This cookbook tag must change to implement the configurations async.
        #
        cookbook_tag = ""
        if 'cookbook_tag' in instance['metadata']['system']['role']:
            cookbook_tag = instance['metadata']['system']['role']['cookbook_tag']

        bootstrap = ""
        if 'bootstrap' in instance['metadata']['system']['role']:
            bootstrap = instance['metadata']['system']['role']['bootstrap']

        instance_id = instance['id']

        # if there are dynamic variables
        if 'environment_raw' in instance['metadata']['infrastructure']:
            environment_raws = instance['metadata']['infrastructure']['environment_raw']
            maccli.logger.debug("[%s] Environment raw: %s" % (instance['servername'], environment_raws))
            maccli.logger.debug("[%s] Instances: %s" % (instance['servername'], other_instances))
            environment_clean, processed = parse_instance_envs(environment_raws, other_instances)
            maccli.logger.debug("[%s] Environment clean: %s" % (instance['servername'], environment_clean))
            metadata_new = {'system': {'role': {'environment': environment_clean}}}
            maccli.logger.debug("[%s] Process %s" % (instance['servername'], processed))
            if processed:
                action = True
                maccli.service.instance.update_configuration(cookbook_tag, bootstrap, instance_id, metadata_new)
                processing_instances = maccli.service.instance.list_by_infrastructure(infrastructure_name, version)
                if quiet:
                    maccli.view.view_generic.show("Applying configuration to %s" % instance['servername'])
                else:
                    maccli.view.view_generic.clear()
                    maccli.view.view_instance.show_instances(processing_instances)
                    maccli.view.view_infrastructure.show_infrastructure_resources(infrastructures, infrastructure_resources_processed)

        # there are not dynamic variables, it can start straight away
        else:
            action = True
            maccli.logger.debug("[%s] Processing ")
            maccli.service.instance.update_configuration(cookbook_tag, bootstrap, instance_id)
            processing_instances = maccli.service.instance.list_by_infrastructure(infrastructure_name, version)
            if quiet:
                maccli.view.view_generic.show("Applying configuration to %s" % instance['servername'])
            else:
                maccli.view.view_generic.clear()
                maccli.view.view_instance.show_instances(processing_instances)
                maccli.view.view_infrastructure.show_infrastructure_resources(infrastructures, infrastructure_resources_processed)

    if action:
        """ Allow all tasks to start """
        time.sleep(3)


def clean_up(instances, on_failure):
    failed = False
    for instance in instances:
        if instance['status'] == "Creation failed" or instance['status'] == "Configuration Error":
            failed = True

    if failed and on_failure is not None:
        maccli.view.view_generic.show_error("Cleaning up")
        for instance in instances:
            if on_failure == MACFILE_ON_FAILURE_DESTROY_OTHERS:
                if not (instance['status'] == "Creation failed" or instance['status'] == "Configuration Error"):
                    maccli.view.view_generic.show_error("Destroying instance %s " % instance['id'])
                    maccli.service.instance.destroy_instance(instance['id'])
            elif on_failure == MACFILE_ON_FAILURE_DESTROY_ALL:
                maccli.view.view_generic.show_error("Destroying instance %s " % instance['id'])
                maccli.service.instance.destroy_instance(instance['id'])
            else:
                maccli.view.view_generic.show_error("ERROR: clean up %s not implemented" % on_failure)

    return failed


def apply_resources(processed_instances, processed_resources, instances, roles, infrastructures, actions, resources, root, quiet):
    """ Apply all the resources that are not instances """
    # If event cannot happen, wait to the next loop
    resources_processed_part = []
    finish = True
    for infrastructure_key in infrastructures:
        infrastructure = infrastructures[infrastructure_key]

        if not any(infrastructure_key in d for d in processed_resources):
            maccli.logger.debug("%s does not exist in processed resources %s" % (infrastructure_key, processed_resources))
            if 'resource' in infrastructure:
                maccli.logger.debug("Type resource")
                log_type = "resource"
                key = infrastructure['resource']
                try:
                    command_raw = resources[key]['create bash']
                except KeyError:
                    raise MacResourceException("Resource %s not found" % key, "Resource %s not found" % key)

                command_destroy = None
                if 'destroy bash' in resources[key].keys():
                    command_destroy = resources[key]['destroy bash']

                if 'ready' in infrastructure:  # wait for the instance to be ready before proceeding
                    is_role_ready = maccli.helper.macfile.is_role_dependencies_ready(infrastructure, processed_instances, infrastructure_key)
                    if is_role_ready:
                        resource_processed, resource_finish = maccli.service.resource.run_raw_command(infrastructure_key, command_raw, log_type, key, instances, roles, infrastructures, actions, processed_resources)
                        if resource_finish:
                            resources_processed_part.append({infrastructure_key: resource_processed})
                            maccli.service.resource.create_resource(root, infrastructure_key, key, command_raw, resource_processed['rc'], resource_processed['stderr'], resource_processed['stdout'], command_destroy, infrastructure)
                            __resource_status(infrastructure_key, resource_processed['cmd'], resource_processed['rc'], resource_processed['stderr'], resource_processed['stdout'],
                                              infrastructures, processed_instances, processed_resources + resources_processed_part)
                        else:
                            finish = False
                            break  # stop processing resources
                    else:
                        finish = False
                        break  # stop processing resources
                else:
                    resource_processed, resource_finish = maccli.service.resource.run_raw_command(infrastructure_key, command_raw, log_type, key, instances, roles, infrastructures, actions, processed_resources)
                    if resource_finish:
                        resources_processed_part.append({infrastructure_key: resource_processed})
                        maccli.service.resource.create_resource(root, infrastructure_key, key, command_raw, resource_processed['rc'], resource_processed['stderr'], resource_processed['stdout'], command_destroy, infrastructure)
                        __resource_status(infrastructure_key, resource_processed['cmd'], resource_processed['rc'], resource_processed['stderr'], resource_processed['stdout'],
                                          infrastructures, processed_instances, processed_resources + resources_processed_part)
                    else:
                        finish = False
                        break  # stop processing resources

            elif 'action' in infrastructure:
                maccli.logger.debug("Type action")
                log_type = "action"
                key = infrastructure['action']
                command_raw = actions[key]['bash']
                resource_processed, resource_finish = maccli.service.resource.run_raw_command(infrastructure_key, command_raw, log_type, key, instances, roles, infrastructures, actions, processed_resources)
                maccli.logger.debug("RESOURCE (ACTION) PROCESSED %s, %s" % (resource_finish, resource_processed))
                if resource_finish:
                    resources_processed_part.append({infrastructure_key: resource_processed})
                    __resource_status(infrastructure_key, resource_processed['cmd'], resource_processed['rc'], resource_processed['stderr'], resource_processed['stdout'],
                                      infrastructures, processed_instances, processed_resources + resources_processed_part)
                else:
                    finish = False
                    break  # stop processing resources
                # TODO ssh not implemented

            elif 'role' in infrastructure:
                maccli.logger.debug("Type role")
                maccli.logger.debug("Processing %s " % infrastructure_key)
                infrastructure_role = infrastructure['role']

                new_environments = []  # parse environments
                if 'environment' in roles[infrastructure_role]["instance create"]:
                    for environment_raw in roles[infrastructure_role]["instance create"]['environment']:
                        environment_clean, environments_processed = maccli.helper.macfile.parse_envs_dict(environment_raw, processed_instances, roles, infrastructures, actions, processed_resources + resources_processed_part)
                        new_environments.append(environment_clean)
                    roles[infrastructure_role]["instance create"]['environment'] = new_environments

                infrastructure_parsed, infrastructure_processed = maccli.helper.macfile.parse_envs_dict(infrastructure, processed_instances, roles, infrastructures, actions, processed_resources + resources_processed_part)

                if infrastructure_processed:
                    maccli.logger.debug("Creating instances for %s " % infrastructure_key)
                    maccli.service.instance.create_instances_for_role(root, infrastructure_parsed, roles, infrastructure_key, quiet)

                # mark as resource processed
                resources_processed_part.append({infrastructure_key: {'stderr': None, 'stdout': None, 'rc': 0, 'cmd': None}})

                continue  # roles are processed independent to resources
            else:
                raise NotImplementedError

    return resources_processed_part, finish


def destroy_resource(resource, instances, resources):

    if resource['cmdDestroy']:

        clean_cmd_destroy = maccli.helper.macfile.parse_envs_destroy(resource, instances, resources)
        maccli.logger.debug("CMD_DESTROY_CLEAN %s" % clean_cmd_destroy)
        rc = None
        stdout = None
        try:
            rc, stdout, stderr = maccli.helper.cmd.run(clean_cmd_destroy)
        except Exception as e:
            maccli.logger.warn("Error executing %s: %s" % (clean_cmd_destroy, e))
            stderr = e[1]

        maccli.service.resource.update_resource(resource, clean_cmd_destroy, rc, stderr, stdout)

        if rc != 0:
            maccli.view.view_generic.showc("Resource %s failed when destroying resource" % resource['name'], RED)
            maccli.view.view_generic.cmd_error(clean_cmd_destroy, rc, stdout, stderr)
        else:
            maccli.view.view_generic.showc("Resource %s has been successfully removed" % resource['name'], GREEN)
        maccli.view.view_generic.show("")
    else:
        maccli.view.view_generic.show("Resource %s skipped" % resource['name'])

    maccli.service.resource.destroy_resource(resource)


def __resource_status(infrastructure_key, command_clean, rc, stderr, stdout, infrastructures, processed_instances, processed_resources):
    """ print summary status of how things are being processed """
    if rc == 0:
        if maccli.quiet:
            maccli.view.view_generic.show("%s executed successfully" % infrastructure_key)
        else:
            maccli.view.view_generic.clear()
            maccli.view.view_instance.show_instances(processed_instances)
            maccli.view.view_infrastructure.show_infrastructure_resources(infrastructures, processed_resources)
        if stdout:
            maccli.logger.debug("STDOUT: %s " % stdout)
        if stderr:
            maccli.logger.debug("STDERR: %s " % stderr)
    else:
        if maccli.quiet:
            maccli.view.view_generic.cmd_error(command_clean, rc, stdout, stderr)
        else:
            maccli.view.view_generic.clear()
            maccli.view.view_instance.show_instances(processed_instances)
            maccli.view.view_infrastructure.show_infrastructure_resources(infrastructures, processed_resources)



import unittest
import subprocess

import mock

from tests.mock_data import *
import maccli.dao.api_auth


class AuthTestCase(unittest.TestCase):
    @mock.patch('maccli.helper.http.send_request')
    def test_auth_authenticate_ok(self, mock_get_auth):
        mock_get_auth.return_value = 200, MOCK_LOGIN_JSON, MOCK_LOGIN_JSON_RAW
        user, apiKey = maccli.dao.api_auth.get_auth(MOCK_USER, MOCK_PASSWORD)
        self.assertEqual(MOCK_USER, user)
        self.assertEqual(MOCK_APIKEY, apiKey)



__author__ = 'tk421'


def metadata_instance(macfile_root, infrastructure_key, role_key, role, infrastructure):
    """
    Generate the json metadata to create an instance
    """
    # version must be string
    macfile_root['version'] = str(macfile_root['version'])
    meta = macfile_root
    meta['macfile_role_name'] = role_key
    meta['macfile_infrastructure_name'] = infrastructure_key
    environment = get_environment(role, infrastructure)
    if environment is not None:
        meta['environment_raw'] = environment
    return meta


def metadata_resource(macfile_root, infrastructure_key, resource_name, infrastructure):
    """
    Generate the json metadata to create an instance
    """
    # version must be string
    macfile_root['version'] = str(macfile_root['version'])
    meta = macfile_root
    meta['macfile_resource_name'] = resource_name
    meta['macfile_infrastructure_name'] = infrastructure_key
    if 'params' in infrastructure:
        meta['macfile_infrastructure_params'] = infrastructure['params']

    return meta


def get_environment(role, infrastructure):
    environment = None

    if 'environment' in role.keys():
        environment = role['environment']

    if 'environment' in infrastructure.keys():
        if environment is not None:
            environment = environment + infrastructure["environment"]
        else:
            environment = infrastructure["environment"]

    return environment


import unittest

import mock

import maccli.service.configuration
from tests.mock_data import *


class AuthTestCase(unittest.TestCase):
    @mock.patch('maccli.dao.api_configuration.get_user_configuration')
    def test_list_configurations(self, mock):
        mock.return_value = (200, MOCK_CONFIGURATION_LIST_JSON)
        json_response = maccli.service.configuration.list_configurations()
        self.assertTrue(mock.called)
        self.assertEqual(json_response, MOCK_CONFIGURATION_LIST_JSON)

    @mock.patch('maccli.dao.api_configuration.search_public_configuration')
    def test_list_configurations(self, mock):
        mock.return_value = (200, MOCK_CONFIGURATION_SEARCH_JSON)
        json_response = maccli.service.configuration.search_configurations(None)
        self.assertTrue(mock.called)
        self.assertEqual(json_response, MOCK_CONFIGURATION_SEARCH_JSON)
